/**
 * AutoFocuser: "static" class which listens to the
 * mouse events to grab the focus when a component is
 * entered.
 * @version 1.0
 * @author Alexandre David
 */

package adavid.awt;

import java.awt.event.*;
import java.awt.Component;

/**
 * AutoFocuser
 * gives focus to a component when mouse enters
 */
public class AutoFocuser implements MouseListener {

  /**
   * one instance, unique
   */
  private static AutoFocuser focuser=new AutoFocuser();

  /**
   * add this focuser as a listener to the component
   */
  public static void add(Component c) {
    c.addMouseListener(focuser);
  }

  /**
   * remove this focuser as a listener from the component
   */
  public static void remove(Component c) {
    c.removeMouseListener(focuser);
  }

  /**
   * MouseListener: used to grab the focus
   */
  public void mouseEntered(MouseEvent e) {
    e.getComponent().requestFocus();
  }

  /**
   * useless functions from the MouseListener interface
   */
  public void mouseClicked(MouseEvent e){}
  public void mousePressed(MouseEvent e){}
  public void mouseReleased(MouseEvent e){}
  public void mouseExited(MouseEvent e){}
  
}
