/**
 * BorderedPanel: a panel with a border!
 * @version 1.1
 * @author Alexandre David
 */

/**
 * 1.0: first version
 * 1.0.1 990905: added getOffset
 * 1.1: 990922 added addComponent(Component) and getPanel(...)
 */

package adavid.awt;

import java.awt.*;
import adavid.awt.*;
import java.awt.event.ItemListener;

/**
 * Bordered Panel
 * a panel bordered with a text and a line
 */
public class BorderedPanel extends Panel {

  /**
   * default values
   */
  public static final int DEFAULT_THICKNESS=3,INC=20,MIDDLE=10;

  NiceLabel label;
  int thickness,x=INC,y=0,xc,yc,wc,dimw,dimh,y0;
  Color color;

  /**
   * default constructor: default color and thickness
   */
  public BorderedPanel(Graphics g,String title) {
    this(g,title,DEFAULT_THICKNESS,Color.blue);
  }
  
  /**
   * default constructor: all default
   */
  public BorderedPanel(Graphics g) {
    this(g,null,DEFAULT_THICKNESS,Color.blue);
  }

  /**
   * default constructor: default thickness
   */
  public BorderedPanel(Graphics g,String title,Color color) {
    this(g,title,DEFAULT_THICKNESS,color);
  }

  /**
   * default constructor: default color
   */
  public BorderedPanel(Graphics g,String title,int thickness) {
    this(g,title,thickness,Color.blue);
  }

  /**
   * the constructor: the Graphics is used for the label,
   * title to display, thickness and color of the border
   */
  public BorderedPanel(Graphics g,String title,int thickness,Color color) {
    this.thickness=thickness;
    this.color=color;
    setLayout(null);
    if (title!=null) {
      label=new NiceLabel(g,title,Label.CENTER,6);
      label.setForeground(color);
      add(label);
      x=label.prefWidth()+INC;
      y=label.prefHeight()/2-thickness/2;
      if (y<0) y=0;
      label.setLocation(y+INC,0);
      x+=y;
      y0=thickness+2*y+2;
    }
    else
      y0=2*thickness;
    dimh=2*y0;
    dimw=dimh;
    xc=y0;
    yc=y0;
    wc=0;
  }
  
  /**
   * returns the used label
   */
  public NiceLabel getLabel() {
    return label;
  }

  /**
   * paint
   */
  public void paint(Graphics g) {
    Dimension dim=getSize();
    g.setColor(color);
    for(int i=0;i<thickness;i++) {
      g.drawLine(y,y+i,INC+y,y+i);
      g.drawLine(x,y+i,dim.width-1-y,y+i);
      g.drawLine(dim.width-1-i-y,y,dim.width-1-i-y,dim.height-1-y);
      g.drawLine(dim.width-1-y,dim.height-1-i-y,y,dim.height-1-i-y);
      g.drawLine(i+y,dim.height-1-y,i+y,y);
    }
  }

    /**
     * add a component
     */
    public void addComponent(Component compo) {
	Dimension dim=compo.getPreferredSize();
	addComponent(compo,dim.width,dim.height);
    }

  /**
   * add a component
   */
  public void addComponent(Component compo,int width,int height) {
    if (compo!=null) {
      add(compo);
      compo.setBounds(xc,yc,width,height);
    }
    yc+=height;
    if (wc<0 && width!=0)
      dimw+=MIDDLE;
    if (wc<width)
      wc=width;
    dimh=Math.max(dimh,yc+y0);
  }

  /**
   * create and add a NiceCheckbox
   */
  public NiceCheckbox addNiceCheckbox(Graphics g,String name) {
    NiceCheckbox check=new NiceCheckbox(g,name);
    addComponent(check,check.prefWidth(),check.prefHeight());
    return check;
  }

  /**
   * a method just for convenience: same as before but may
   * initialize a check box group and an item listener..
   */
  public NiceCheckbox addNiceCheckbox(Graphics g,String name,
				      CheckboxGroup group,ItemListener listener) {
    NiceCheckbox check=new NiceCheckbox(g,name);
    addComponent(check,check.prefWidth(),check.prefHeight());
    if (group!=null)
      check.setCheckboxGroup(group);
    if (listener!=null)
      check.addItemListener(listener);
    return check;
  }

  /**
   * break the column
   */
  public void breakColumn() {
    xc+=wc+MIDDLE;
    yc=y0;
    dimw+=wc;
    wc=-1;
  }

  /**
   * preferred size
   */
  public Dimension getPreferredSize() {
    return new Dimension(dimw,dimh);
  }

  /**
   * preferred Width
   */
  public int prefWidth() {
    return dimw;
  }
  
  /**
   * preferred Height
   */
  public int prefHeight() {
    return dimh;
  }

    /**
     * offset from border
     */
    public int getOffset() {
	return y0;
    }

    /**
     * get a simple panel with the given checkboxes,
     * may have more configuration options in the future
     * @param g a valid graphics object
     * @param color the color of the border
     * @param title the title of the panel
     * @param list the list of components to put in
     * the panel
     */
    public static BorderedPanel getPanel(Graphics g,Color color,String title,Component[] list) {
	BorderedPanel panel=new BorderedPanel(g,title,color);
	for(int i=0;i<list.length;i++)
	    panel.addComponent(list[i]);
	panel.breakColumn();
	panel.setSize(panel.dimw,panel.dimh);
	return panel;
    }
}




