/**
 * DefaultTNodeRenderer
 * the default implementation of a TNodeRenderer
 * @version 1.0
 * @author Alexandre David
 */

package adavid.awt;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Component;

/**
 * default tree renderer
 */
public class DefaultTNodeRenderer implements TNodeRenderer {

  /**
   * a shared instance for this renderer, since its only function
   * is to draw a node
   */
  public final static DefaultTNodeRenderer INSTANCE=new DefaultTNodeRenderer();
  /**
   * constant controling the size of the drawing
   */
  public final static int SIZE=8;

  /**
   * coordinates for the drawing
   */
  static final int[] pictx={SIZE,SIZE/2,0};
  static final int[] picty={0,SIZE,0};

  /**
   * a protected constructor in order not to use it : use the shared instance
   * protected so that the class could be extended to some customized renderer
   */
  protected DefaultTNodeRenderer() {}

  /**
   * basic drawing : a triangle ; note that the color is set
   * by TNode
   */
  public void paint(TNode node,Graphics g,boolean expanded,boolean selected,boolean leaf) {
    if (!leaf) {
      if (expanded)
	g.fillPolygon(pictx,picty,3);
      else
	g.fillPolygon(picty,pictx,3);
    }
  }

  /**
   * the height taken
   */
  public int getHeight() {
    return SIZE+1;
  }

  /**
   * the width taken
   */
  public int getWidth() {
    return SIZE+2;
  }

}









