/**
 * NiceButton: a little improved button, with
 * bold font, autofocusing and cursor changed +
 * better controlled size.
 * @version 1.0
 * @author Alexandre David
 */

package adavid.awt;

import java.awt.*;

/**
 * a nicer Button with mouse focus
 */
public class NiceButton extends Button {

  int w,h,border;
  Graphics g;

  /**
   * default constructor: needs a graphics to change the font
   */
  public NiceButton(Graphics g,String name) {
    this(g,name,10);
  }

  /**
   * constructor: needs a graphics to change the font,
   * the border is the supplementary border to use
   */
  public NiceButton(Graphics g,String name,int border) {
    super(name);
    setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    AutoFocuser.add(this);
    this.g=g;
    this.border=border;

    if (g!=null) {
      Font f=g.getFont();
      Font font=new Font(f.getName(),Font.BOLD,f.getSize());
      FontMetrics fm=g.getFontMetrics(font);
      setFont(font);
      setSize(w=fm.stringWidth(name)+2*border,
	      h=fm.getHeight()+border);
    }
  }

  /**
   * overrides setText in order to update w and h
   */
  public synchronized void setText(String text) {
    super.setLabel(text);
    Font font=getFont();
    if (g!=null && font!=null) {
      FontMetrics fm=g.getFontMetrics(font);
      w=fm.stringWidth(text)+2*border;
      h=fm.getHeight()+border;
    }
  }

  /**
   * overrides setFont to update w and h
   */
  public void setFont(Font font) {
    super.setFont(font);
    if (g!=null && font!=null) {
      FontMetrics fm=g.getFontMetrics(font);
      w=fm.stringWidth(getLabel())+2*border;
      h=fm.getHeight()+border;
    }
  }

  /**
   * returns the preferred size
   */
  public Dimension getPreferredSize() {
    if (w!=0 && h!=0)
      return new Dimension(w,h);
    else
      return super.getPreferredSize();
  }

  /**
   * returns preferred dimensions (w,h)
   */
  public int prefWidth() {
    if (w!=0)
      return w;
    else
      return super.getPreferredSize().width;
  }
  public int prefHeight() {
    if (h!=0)
      return h;
    else
      return super.getPreferredSize().height;
  }

}
