/**
 * NiceCheckbox enhanced awt component: + auto focusing
 * and nice font + cursor changed
 * @author Alexandre David
 * @version 1.0
 */

package adavid.awt;

import java.awt.*;

/**
 * a nicer checkbox with mouse focus
 */
public class NiceCheckbox extends Checkbox {

  int w,h;

  public final static Cursor HAND=Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);

  /**
   * default constructors: needs a graphics to change the font
   */
  public NiceCheckbox(Graphics g,String name) {
    this(g,name,false,null);
  }
  public NiceCheckbox(Graphics g,String name,boolean state) {
    this(g,name,state,null);
  }

  /**
   * constructor: needs a graphics to change the font,
   * the border is the supplementary border to use
   */
  public NiceCheckbox(Graphics g,String name,boolean state,CheckboxGroup group) {
    super(name,state,group);
    setCursor(HAND);
    AutoFocuser.add(this);

    if (g!=null) {
      Font f=g.getFont();
      Font font=new Font(f.getName(),Font.BOLD,f.getSize());
      FontMetrics fm=g.getFontMetrics(font);
      setFont(font);
      setSize(w=fm.stringWidth(name)+30,
	      h=fm.getHeight()+10);
    }
  }

  /**
   * make nice a component + cursor
   */
  public static void makeNice(Graphics g,Component c) {
    if (g!=null) {
      Font f=g.getFont();
      c.setFont(new Font(f.getName(),Font.BOLD,f.getSize()));
    }
    AutoFocuser.add(c);
    c.setCursor(HAND);
  }

  /**
   * returns the preferred size
   */
  public Dimension getPreferredSize() {
    return new Dimension(w,h);
  }

  /**
   * returns preferred dimensions (w,h)
   */
  public int prefWidth() {
    return w;
  }
  public int prefHeight() {
    return h;
  }

}
