/**
 * NiceLabel: a better looking label, still based
 * on AWT, it looks like swing labels, just bold font!
 * and the size is better controlled.
 * @version 1.0
 * @author Alexandre David
 */

package adavid.awt;

import java.awt.*;

/**
 * a nicer label with bold font
 */
public class NiceLabel extends Label {

  int w=0,h=0,border=0;
  Graphics g;

  /**
   * default constructor: needs a graphics to change the font
   */
  public NiceLabel(Graphics g,String name) {
    this(g,name,LEFT,10);
  }

  /**
   * default constructor 2: can change the alignment
   */
  public NiceLabel(Graphics g,String name,int align) {
    this(g,name,align,10);
  }

  /**
   * constructor: needs a graphics to change the font
   * and the border gives extra border
   */
  public NiceLabel(Graphics g,String name,int align,int border) {
    super(name,align);
    this.border=border;
    this.g=g;
    if (g!=null) {
      Font f=g.getFont();
      Font font=new Font(f.getName(),Font.BOLD,f.getSize());
      FontMetrics fm=g.getFontMetrics(font);
      super.setFont(font);
      setSize(w=fm.stringWidth(name)+border,
	      h=fm.getHeight());
    }
  }
  
  /**
   * make nice, just a new font
   */
   public static void makeNice(Graphics g,Component c) {
    if (g!=null) {
      Font f=g.getFont();
      c.setFont(new Font(f.getName(),Font.BOLD,f.getSize()));
    }
  }

  /**
   * overrides setText to update w and h
   */
  public synchronized void setText(String text) {
    super.setText(text);
    Font font=getFont();
    if (g!=null && font!=null) {
      FontMetrics fm=g.getFontMetrics(font);
      w=fm.stringWidth(text)+border;
      h=fm.getHeight();
    }
  }

  /**
   * overrides setFont in order to update h and w
   */
  public void setFont(Font font) {
    super.setFont(font);
    if (g!=null && font!=null) {
      FontMetrics fm=g.getFontMetrics(font);
      w=fm.stringWidth(getText())+border;
      h=fm.getHeight();
    }
  }

  /**
   * returns the preferred size
   */
  public Dimension getPreferredSize() {
    if (h!=0 && w!=0)
      return new Dimension(w,h);
    else
      return super.getPreferredSize();
  }

  /**
   * returns preferred dimensions (w,h)
   */
  public int prefWidth() {
    if (w!=0)
      return w;
    else
      return super.getPreferredSize().width;
  }
  public int prefHeight() {
    if (h!=0)
      return h;
    else
      return super.getPreferredSize().height;
  }

}
