/**
 * PopupManager: a "static" class to attach and dettach
 * popups to components, the advantage is that we do not
 * have to manage the popup open.
 * @version 1.1
 * @author Alexandre David
 */
package adavid.awt;

/**
 * version history:
 * 1.0: first version
 * 1.1 990829: use swing 1.1.1
 */

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.JPopupMenu;
import javax.swing.JComponent;

/**
 * PopupManager
 * attach, dettach popup menus to components and
 * manages the open/close
 */
public class PopupManager extends MouseAdapter {

  /**
   * the only instance of the manager
   */
  private static PopupManager manager=new PopupManager();

  /**
   * the table key=component,value=popup to get the right
   * popup menu attached to a component
   */
  private static Hashtable table=new Hashtable();

  /**
   * attach a PopupMenu to a component
   */
  public static void attachPopupMenu(Component compo,PopupMenu popup) {
    if (popup!=null && compo!=null) {
      compo.addMouseListener(manager);
      synchronized(table) {
	table.put(compo,popup);//only one popup per component
      }
    }
  }

  /**
   * attach a JPopupMenu to a JComponent
   */
  public static void attachJPopupMenu(JComponent compo,JPopupMenu popup) {
    if (popup!=null && compo!=null) {
      popup.setInvoker(compo);
      compo.addMouseListener(manager);
      synchronized(table) {
        table.put(compo,popup);
      }
    }
  }
 
  /**
   * detach a PopupMenu from a component
   */
  public void dettachPopup(Component compo) {
    if (compo!=null) {
      Object popup;
      synchronized(table) {
	popup=table.remove(compo);
      }
      compo.removeMouseListener(manager);
    }
  }
  
  /**
   * Invoked when a mouse button has been pressed on a component.
   * opens the popup menu
   */
  public void mousePressed(MouseEvent e) {
    if (e.isPopupTrigger()) {
      Component source=e.getComponent();
      Object popup;
      synchronized(table) {
        popup=table.get(source);
      }
      //2 cases are necessary since a JPopupMenu is not an
      //extension of PopupMenu!
      //it is possible to use this class without having swing:
      //if JPopupMenu is never needed for testing, it will never be
      //loaded!
      if (popup instanceof PopupMenu) //null is not an instanceof PopupMenu
        ((PopupMenu)popup).show(source,e.getX(),e.getY());
      else if (popup instanceof JPopupMenu)
	((JPopupMenu)popup).show(source,e.getX(),e.getY());
      //note that the virtual machine loads classes when needed which
      //implies that this code may be used even without swing at all
      //and it will work all right because the second line will never be
      //executed.
      //I should have an else... but else what? if another kind of
      //object is present then it is an inconsistency somewhere,
      //and on the other hand, the signature of mousePressed is so
      //that it should not send any exception...
    }
  }
}





