/**
 * JTextAreaOutputStream: output stream which redirects
 * the output to a Jtext area. It's a bit deprecated
 * but very useful if you want to redirect System.out
 * to a PrintStream(thiskindofclass).
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.2 990829: use swing 1.1.1
 * 1.1.1 990801: fixed the internal buffer
 *      a stupid unefficiency (result of a copy-paste)
 *	and add synchronized
 * 1.1: added internal buffer, more
 *      efficient than wrapping this class
 *      with a BufferedOutputStream
 * 1.0: first version
 */

package adavid.io;

import java.io.*;
import javax.swing.JTextArea;

/**
 * JTextAreaOutputStream
 * output stream JTextArea (from JStringOutputStream)
 * as JTextArea is slow, it could be nice to use this class
 * inside a BufferedOutputStream to enhance performance
 */
public class JTextAreaOutputStream extends OutputStream {

  /**
   * where to output
   */
  JTextArea jta=null;

  /**
   * to use buffer capability
   */
  StringBuffer buffer=null;

  /**
   * and store its size locally
   */
  int bufsize=0;

  /**
   * constructor: to a J text area
   */
  public JTextAreaOutputStream(JTextArea ta) {
    jta=ta;
  }

  /**
   * constructor: the text area and size of the
   * buffer to use (none by default)
   */
  public JTextAreaOutputStream(JTextArea ta,int size) {
    jta=ta;
    if (size>1) //ridiculous to bufferize 1
      buffer=new StringBuffer(bufsize=size);
  }

  /**
   * implements the stream: writes the byte to the string buffer
   */
  public synchronized void write(int b) {
    if (buffer!=null) {
      if (buffer.length()>=bufsize){ //overflow
	if (jta!=null)
	  jta.append(buffer.toString()); //flush
        buffer.setLength(0);
      }
      buffer.append((char)b);
    }
    else if (jta!=null)
      jta.append(String.valueOf((char)b)); //stupid, but no append(char)
  }

  /**
   * returns the written string
   */
  public synchronized String toString() {
    //inline flush and we avoid a test by this way
    if (buffer!=null && buffer.length()>0 && jta!=null) {
      jta.append(buffer.toString());
      buffer.setLength(0);
      return jta.getText();
    }
    else if (jta!=null)
      return jta.getText();
    else
      return null;
  }

  /**
   * overrides OutputStream.flush()
   */
  public synchronized void flush() {
    if (buffer!=null && buffer.length()>0) {
      if (jta!=null)
        jta.append(buffer.toString());
      buffer.setLength(0);
    }
  }

}
