/**
 * JTextAreaWriter: writer which redirects
 * the output to a text area.
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.0 990801: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.io;

import java.io.*;
import javax.swing.JTextArea;

/**
 * TextAreaWriter
 * Writer to a TextArea
 */
public class JTextAreaWriter extends Writer {

  /**
   * where to output
   */
  JTextArea textArea=null;

  /**
   * to use buffer capability
   */
  StringBuffer buffer=null;

  /**
   * and store its size locally
   */
  int bufsize=0;

  /**
   * constructor: output to a text area
   */
  public JTextAreaWriter(JTextArea ta) {
    textArea=ta;
  }

  /**
   * constructor: the text area and size of the
   * buffer to use (none by default)
   */
  public JTextAreaWriter(JTextArea ta,int size) {
    textArea=ta;
    if (size>1) //ridiculous to bufferize 1 (or 2...)
      buffer=new StringBuffer(bufsize=size);
  }

  /**
   * implements the writer: writes the byte to the string buffer
   */
  public synchronized void write(char[] cbuf,int off,int len) {
    if (buffer!=null) {
      if (buffer.length()>=bufsize){ //overflow
	if (textArea!=null)
	  textArea.append(buffer.toString()); //flush
        buffer.setLength(0);
      }
      buffer.append(cbuf,off,len);
    }
    else if (textArea!=null)
      textArea.append(new String(cbuf,off,len));
  }

  /**
   * returns the written string
   */
  public synchronized String toString() {
    //inline flush and we avoid a test by this way
    if (buffer!=null && buffer.length()>0 && textArea!=null) {
      textArea.append(buffer.toString());
      buffer.setLength(0);
      return textArea.getText();
    }
    else if (textArea!=null)
      return textArea.getText();
    else
      return null;
  }

  /**
   * implements flush
   */
  public synchronized void flush() {
    if (buffer!=null && buffer.length()>0) {
      if (textArea!=null)
        textArea.append(buffer.toString());
      buffer.setLength(0);
    }
  }

    /**
     * implements close
     */
    public void close() {
	//nothing
    }

}

