/**
 * TextAreaOutputStream: output stream which redirects
 * the output to a text area. It's a bit deprecated
 * but very useful if you want to redirect System.out
 * to a PrintStream(thiskindofclass).
 * @version 1.1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.1.1 990801: fixed the internal buffer
 *      a stupid unefficiency (result of a copy-paste)
 *      and add synchronized
 * 1.1: added internal buffer, more
 *      efficient than wrapping this class
 *      with a BufferedOutputStream
 * 1.0: first version
 */

package adavid.io;

import java.io.*;
import java.awt.TextArea;

/**
 * TextAreaOutputStream
 * output stream to a TextArea
 */
public class TextAreaOutputStream extends OutputStream {

  /**
   * where to output
   */
  TextArea textArea=null;

  /**
   * to use buffer capability
   */
  StringBuffer buffer=null;

  /**
   * and store its size locally
   */
  int bufsize=0;

  /**
   * constructor: output to a text area
   */
  public TextAreaOutputStream(TextArea ta) {
    textArea=ta;
  }

  /**
   * constructor: the text area and size of the
   * buffer to use (none by default)
   */
  public TextAreaOutputStream(TextArea ta,int size) {
    textArea=ta;
    if (size>1) //ridiculous to bufferize 1 (or 2...)
      buffer=new StringBuffer(bufsize=size);
  }

  /**
   * implements the stream: writes the byte to the string buffer
   */
  public synchronized void write(int b) {
    if (buffer!=null) {
      if (buffer.length()>=bufsize){ //overflow
	if (textArea!=null)
	  textArea.append(buffer.toString()); //flush
        buffer.setLength(0);
      }
      buffer.append((char)b);
    }
    else if (textArea!=null)
      textArea.append(String.valueOf((char)b));
  }

  /**
   * returns the written string
   */
  public synchronized String toString() {
    //inline flush and we avoid a test by this way
    if (buffer!=null && buffer.length()>0 && textArea!=null) {
      textArea.append(buffer.toString());
      buffer.setLength(0);
      return textArea.getText();
    }
    else if (textArea!=null)
      return textArea.getText();
    else
      return null;
  }

  /**
   * overrides OutputStream.flush()
   */
  public synchronized void flush() {
    if (buffer!=null && buffer.length()>0) {
      if (textArea!=null)
        textArea.append(buffer.toString());
      buffer.setLength(0);
    }
  }

}




