/**
 * ObjectMethod
 * This is to remember methods with their invoker objects.
 * Invoked by ActionManager
 * @version 1.0
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 */

package adavid.reflect;

import java.lang.reflect.*;

/**
 * ObjectMethod
 * this is a method with its Object invoker, the accessed methods may only be public
 */
public class ObjectMethod {
  
    /**
     * remember the method to call
     */
  Method method;

    /**
     * the invoker object
     */
  Object invoker;

    /**
     * contructor:
     * @param invoker the object which invokes the method
     * @param name the name of the method to be called
     * @param args the signature(type) of the the arguments
     */
  public ObjectMethod(Object invoker,String name,Class[] args) throws NoSuchMethodException, SecurityException {
    method=invoker.getClass().getMethod(name,args);
    this.invoker=invoker;
  }

    /**
     * invoke the method, just with the arguments
     */
  public Object invoke(Object[] args) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
    return method.invoke(invoker,args);
  }

    /**
     * overload of Object.toString()
     */
  public String toString() {
    return method.toString();
  }

}
