/**
 * SortableConstructor
 * a wrapper to sort constructors
 * @version 1.0
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 */

package adavid.reflect;

import adavid.util.*;
import java.lang.reflect.*;

/**
 * SortableConstructor
 * to sort constructors: by name and argument
 *
 *
 * same principle as ObjectString, may return
 * the Constructor as well (by a cast)
 */
public class SortableConstructor extends ObjectString {

  /**
   * constructors
   */
  public SortableConstructor(Constructor constructor) {
    super(constructor);
  }
  //not public because the sort string has a meaning: it is not whatsoever!
  SortableConstructor(Constructor constructor,
			     String sortString) {
    super(constructor,sortString,ClassReflect.getModifiers(constructor).append(sortString).toString());
  }

  /**
   * returns the typed object
   * cast error if setObject used with a bad object
   */
  public Constructor getConstructor() {
    return (Constructor)getObject();
  }

  /**
   * returns the source of this constructor in the given source string
   */
  public String getSource(String classSource) {
    return getSource(classSource,getConstructor());
  }

  /**
   * returns the position of this constructor in the given source string
   */
  public Interval getSourcePos(String classSource) {
    return getSourcePos(classSource,getConstructor());
  }

  /**
   * returns the position of the constructor in the given source argument
   */
  public static Interval getSourcePos(String classSource,Constructor constructor) {
    return JavaFinder.getSourcePos(ResourceLoader.shortName(constructor.getDeclaringClass()),
				   ClassReflect.getNames(constructor.getParameterTypes()),
				   classSource);
  }

  /**
   * returns the source of the constructor in the given argument
   */
  public static String getSource(String classSource,Constructor constructor) {
    return getSourcePos(classSource,constructor).substring(classSource);
  }

}
