/**
 * SortableMethod
 * a wrapper for the methods so that they can
 * be sorted + their source can be accessed easily
 * @version 1.0
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 */

package adavid.reflect;

import adavid.util.*;
import java.lang.reflect.*;

/**
 * SortableMethod
 * to sort methods: by type, name and argument
 *
 * same principle as ObjectString, may return
 * the Method as well (by a cast)
 */
public class SortableMethod extends ObjectString {

  /**
   * constructors
   */
  public SortableMethod(Method method) {
    super(method);
  }
  //not public because the sort string has a meaning: it is not whatsoever!
  SortableMethod(Method method,String sortString) {
    super(method,sortString,ClassReflect.getModifiers(method).append(sortString).toString());
  }

  /**
   * returns the typed object,
   * cast error if setObject used with a bad object
   */
  public Method getMethod() {
    return (Method)getObject();
  }

  /**
   * returns the source of this constructor in the given source string
   */
  public String getSource(String classSource) {
    return getSource(classSource,getMethod());
  }

  /**
   * returns the position of this constructor in the given source string
   */
  public Interval getSourcePos(String classSource) {
    return getSourcePos(classSource,getMethod());
  }

  /**
   * returns the position of a method in the given source string
   */
  public static Interval getSourcePos(String classSource,Method method) {
    return JavaFinder.getSourcePos(method.getName(),
				   ClassReflect.getNames(method.getParameterTypes()),
				   classSource);
  }

  /**
   * returns the source of the method in the given source string
   */
  public static String getSource(String classSource,Method method) {
    return getSourcePos(classSource,method).substring(classSource);
  }

}
