/**
 * BToolBar
 * this toolbar keeps a BorderLayout layout, and
 * corrects the swing's Box layout when it is changed
 * because we do not want it!!!
 * One could say: just set BorderLayout to the toolbar,
 * BUT I answer java-swing changes the layout automatically
 * and sometimes we get a copy of the original BorderLayout
 * and sometimes the BoxLayout is restored...
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.swing;

import javax.swing.JToolBar;
import java.awt.BorderLayout;
import java.awt.Component;
import java.util.Hashtable;

public class BToolBar extends JToolBar {

  /**
   * hash table to store one component per position
   */
  Hashtable hash=new Hashtable();

  /**
   * the component will be put in a good
   * toolbar with the appropriate BorderLayout
   */
  public BToolBar(Component component) {
    this();
    add(BorderLayout.CENTER,component);
  }

  public BToolBar() {
    super();
    setLayout(new BorderLayout());
  }

  /**
   * add a component: overrides Container.add(..)
   */
  public Component add(Component compo,String where) {
    return add(where,compo);
  }
  public Component add(String where,Component compo) {
    if (compo!=null) {
      hash.put(where,compo);
      return super.add(where,compo);
    }
    else
      return null;
  }

  /**
   * we overwrite doLayout, to change the layout manager
   * when java gently changes it without telling anything!!
   * So.. we check the layout and reset it whenever it changes!
   */
  public void doLayout() {
    if (!(getLayout() instanceof BorderLayout)) {
      setLayout(new BorderLayout());
      add(BorderLayout.CENTER,(Component)hash.get(BorderLayout.CENTER));
      add(BorderLayout.NORTH,(Component)hash.get(BorderLayout.NORTH));
      add(BorderLayout.SOUTH,(Component)hash.get(BorderLayout.SOUTH));
      add(BorderLayout.WEST,(Component)hash.get(BorderLayout.WEST));
      add(BorderLayout.EAST,(Component)hash.get(BorderLayout.EAST));
    }
    super.doLayout();
  }
}
