/**
 * ButtonMenuItem
 * A simple menu item linked with a button to
 * keep consistent state with the linked button.
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.swing;

import javax.swing.JMenuItem;
import javax.swing.Icon;
import javax.swing.AbstractButton;
import java.awt.event.*;


public class ButtonMenuItem extends JMenuItem implements ActionListener {

  /**
   * the peer button, to be updated (selected,enabled)
   */
  AbstractButton button;

  /**
   * contructors: just register the peer button
   */
  public ButtonMenuItem(AbstractButton button) {
    this(null,null,button);
  }
  public ButtonMenuItem(Icon icon,AbstractButton button) {
    this(null,icon,button);
  }
  public ButtonMenuItem(String name,AbstractButton button) {
    this(name,null,button);
  }
  public ButtonMenuItem(String name,Icon icon,AbstractButton button) {
    super(name,icon);
    this.button=button;
    addActionListener(this);
  }
  
  /**
   * keep the peer button in the same state
   */
  public void setEnabled(boolean b) {
    super.setEnabled(b);
    if (button!=null && (b!=button.isEnabled()))
      button.setEnabled(b);
  }

  /**
   * keep the peer button in the same state
   */
  public void setSelected(boolean b) {
    super.setSelected(b);
    if (button!=null && (b!=button.isSelected()))
      button.setSelected(b);
  }

  /**
   * returns the associated button
   */
  public AbstractButton getButton() {
    return button;
  }

  /**
   * changes the associated button
   */
  public void setButton(AbstractButton button) {
    this.button=button;
    button.setEnabled(isEnabled());
    button.setSelected(isSelected());
  }

  /**
   * select peers when pressed
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(this))
      button.setSelected(true);
  }

}
