/**
 * FocusableBorder
 * A listener which shows/hides the border of a
 * button if the cursor is in/out-side the button.
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.0: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.swing;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;


public class FocusableBorder implements MouseListener,ChangeListener,MouseMotionListener {
  
  /**
   * the only instance of this class
   */
  private static FocusableBorder instance=new FocusableBorder();

  /**
   * add a button to be monitored
   */
  public static void add(AbstractButton button) {
    button.setBorderPainted(false);
    button.addMouseListener(instance);
    button.addMouseMotionListener(instance);
    button.addChangeListener(instance);
  }
  
  /**
   * remove a button (remove the instance from the button's listener list)
   */
  public static void remove(AbstractButton button) {
    button.removeMouseListener(instance);
    button.removeMouseMotionListener(instance);
    button.removeChangeListener(instance);
  }

  /**
   * show the border if the button is enabled
   */
  public void mouseEntered(MouseEvent e) {
    AbstractButton button=(AbstractButton)e.getSource();
    if (button.isEnabled() && !button.isBorderPainted())
      button.setBorderPainted(true);
  }

  /**
   * hide the border
   */
  public void mouseExited(MouseEvent e) {
    AbstractButton button=(AbstractButton)e.getSource();
    if (button.isBorderPainted() && !button.isSelected())
      button.setBorderPainted(false);
  }

  /**
   * if the state changes, then an update may be needed
   */
  public void stateChanged(ChangeEvent e) {
    AbstractButton button=(AbstractButton)e.getSource();
    if ((!button.isEnabled() || !button.isSelected()) && button.isBorderPainted())
      button.setBorderPainted(false);
    else if (button.isSelected() && button.isEnabled() && !button.isBorderPainted())
      button.setBorderPainted(true);
  }

  /**
   * to correct the set false if it not selected and we press or release the button
   */
  public void mouseReleased(MouseEvent e) {
    mouseEntered(e);
  }
  public void mousePressed(MouseEvent e) {
    mouseEntered(e);
  }

  /**
   * to correct when the mouse is dragged out/in
   */
  public void mouseDragged(MouseEvent e) {
    mouseEntered(e);
  }

  /**
   * useless methods of listeners
   */
  public void mouseClicked(MouseEvent e) {}
  public void mouseMoved(MouseEvent e) {}
}
