/**
 * JViewer: a simple viewer for html text
 * @author Alexandre David
 * @version 2.0
 */

/**
 * history:
 * 1.0: 991011
 * 2.0: 991107
 *      - construction of the Document from source file
 *        directly, without HTML intermediate
 *      991204
 *      - completed tags management and added navigation buttons
 *      991205
 *      - swing 1.1.1 window blit enabled
 */

package adavid.swing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import adavid.util.*;
import adavid.awt.*;
import adavid.reflect.*;
import java.lang.reflect.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.text.*;

public class JViewer extends JFrame implements ActionListener,MouseListener,MouseMotionListener {

    /**
     * objects relative to the java text viewer
     */
    private Element curElem=null;
    private Object anchorRef=null;
    private JTextPane textPane;
    private JScrollPane scroller;
    private Cursor defaultCursor;
    public static final Cursor HANDCURSOR=Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
    public static final String ANCHOR="@";

    /**
     * cache textPane because it is slow, I'll see
     * concerning the memory consumption, not static because
     * it is attached to a particular window
     */
    private Hashtable cache=new Hashtable();

    /**
     * color constants
     */
    public static final Color
	STRING_COLOR=new Color(0xF0,0x90,0x24),
	COMMENT1_COLOR=new Color(0xFF,0x0,0x70),
	COMMENTx_COLOR=new Color(0xDD,0x50,0x70),
	MODIF_COLOR=new Color(0x0,0xDD,0x88),
	PUBLIC_COLOR=new Color(0x0,0xBB,0x00),
	PROTECTED_COLOR=new Color(0x0,0x0,0xBB),
	PRIVATE_COLOR=new Color(0xBB,0x0,0x0),
	THROW_COLOR=new Color(0xBB,0x0,0x99),
	FINAL_COLOR=new Color(0x0,0x88,0x77),
	KEY_COLOR=Color.blue.darker(),
	TYPE_COLOR=Color.green.darker(),
	SYMBOL_COLOR=Color.blue,
	LOOP_COLOR=new Color(0xAA,0x0,0xAA);

    public static final char[][][] KEYID={
        {{'b','s','t','r','a','c','t',(char)0x82}},
        {{'y','t','e',(char)0x43},
         {'o','o','l','e','a','n',(char)0x73},
         {'r','e','a','k',(char)0x58}},
        {{'a','t','c','h',(char)0x51},
         {'h','a','r',(char)0x43},
         {'l','a','s','s',(char)0x57},
         {'a','s','e',(char)0x48},
         {'o','n','t','i','n','u','e',(char)0x88}},
        {{'o','u','b','l','e',(char)0x63},
         {'e','f','a','u','l','t',(char)0x78},
         {'o',(char)0x28}},
        {{'x','t','e','n','d','s',(char)0x77},
         {'l','s','e',(char)0x47}},
        {{'i','n','a','l','l','y',(char)0x71},
         {'i','n','a','l',(char)0x52},
         {'l','o','a','t',(char)0x53},
         {'o','r',(char)0x38},
         {'a','l','s','e',(char)0x59}},
        null,
        null,
        {{'n','t',(char)0x33},
         {'m','p','l','e','m','e','n','t','s',(char)0xa7},
         {'f',(char)0x27},
         {'m','p','o','r','t',(char)0x67},
         {'n','s','t','a','n','c','e','o','f',(char)0xa7}},
        null,
        null,
        {{'o','n','g',(char)0x43}},
        null,
        {{'a','t','i','v','e',(char)0x60},
         {'e','w',(char)0x37},
         {'u','l','l',(char)0x49}},
        null,
        {{'u','b','l','i','c',(char)0x64},
         {'r','o','t','e','c','t','e','d',(char)0x95},
         {'r','i','v','a','t','e',(char)0x76},
         {'a','c','k','a','g','e',(char)0x77}},
        null,
        {{'e','t','u','r','n',(char)0x67}},
        {{'t','a','t','i','c',(char)0x60},
         {'y','n','c','h','r','o','n','i','z','e','d',(char)0xc0},
         {'h','o','r','t',(char)0x53},
         {'w','i','t','c','h',(char)0x68},
         {'u','p','e','r',(char)0x59}},
        {{'r','a','n','s','i','e','n','t',(char)0x90},
         {'h','r','o','w',(char)0x51},
         {'h','r','o','w','s',(char)0x61},
         {'r','y',(char)0x31},
         {'r','u','e',(char)0x49},
         {'h','i','s',(char)0x49}},
        {{'n','t','i','l',(char)0x58}},
        {{'o','l','a','t','i','l','e',(char)0x80},
         {'o','i','d',(char)0x43}},
        {{'h','i','l','e',(char)0x58}}};

    public static char[]
	_SINCE={'i','n','c','e'},
	_RETURN={'e','t','u','r','n'},
	_PARAM={'a','r','a','m'},
	_DEPRECATED={'e','p','r','e','c','a','t','e','d'},
	_EXCEPTION={'x','c','e','p','t','i','o','n'},
        _PRE={'p','r','e'},
        _CODE={'c','o','d','e'};
    
    /**
     * Buttons of the window
     */
    private JButton close,find,open,back,next;

    /**
     * history management
     */
    private HistoryEntry[] history;
    private int where,size;

    /**
     * text field for search
     */
    private JTextField input;

    /**
     *  the assistant which knows all
     */
    private Assistant master;

    /**
     * to remember the name of this class
     * to make automatic completion to link of the
     * type #somewhere instead of classname#somewhere
     */
    private String className;

    /**
     * to know if this window is reused or not with the multiple
     * window option
     */
    private boolean multiple;

    /**
     * constructor
     */
    public JViewer(String className,String title,String content,Assistant master) {
	super(title);
	this.master=master;
	this.className=className;
	Container pane=getContentPane();
	pane.setLayout(new BorderLayout());
	JPanel north=new JPanel(new BorderLayout());
	JPanel south=new JPanel(new FlowLayout(FlowLayout.CENTER,5,5));
	JPanel northeast=new JPanel(new FlowLayout(FlowLayout.CENTER,2,1));
	JPanel northwest=new JPanel(new FlowLayout(FlowLayout.CENTER,2,1));
	scroller=new JScrollPane();
	pane.add(north,BorderLayout.NORTH);
	pane.add(south,BorderLayout.SOUTH);
	pane.add(scroller,BorderLayout.CENTER);
	
	StyleContext context=new StyleContext();
	DefaultStyledDocument doc=new DefaultStyledDocument(context);
	initDoc(doc,context,content);
	textPane=new JTextPane(doc);
	textPane.setEditable(false);
	textPane.setCaretPosition(0);

	where=size=0;
	history=new HistoryEntry[10];
	if (content!=null && content.length()>0) {
	    history[0]=new HistoryEntry(className,title,textPane);
	    cache.put(className+"#"+title+master.getSig(),textPane);
	    multiple=true;
	} else multiple=false;

	scroller.getViewport().add(textPane);
	scroller.getViewport().putClientProperty("EnableWindowBlit",master.WBLIT);
	north.add(northwest,BorderLayout.WEST);
	north.add(northeast,BorderLayout.EAST);
	south.add(close=new JButton(master.closemsg));
	northwest.add(back=new JButton("<<"));
	northwest.add(next=new JButton(">>"));
	northeast.add(find=new JButton(master.findmsg));
	northeast.add(open=new JButton(master.openstr));
	north.add(input=new JTextField(),BorderLayout.CENTER);
	close.addActionListener(this);
	find.addActionListener(this);
	open.addActionListener(this);
	input.addActionListener(this);
	back.setEnabled(false);
	next.setEnabled(false);
	back.addActionListener(this);
	next.addActionListener(this);

	textPane.setSelectionColor(Color.darkGray);
	textPane.addMouseMotionListener(this);
	textPane.addMouseListener(this);

	defaultCursor=getCursor();
	//pack();
	//Dimension dim=getPreferredSize();
	//if (dim.width>DialogFactory.MAX_WIDTH)
	//    dim.width=DialogFactory.MAX_WIDTH;
	//if (dim.height>DialogFactory.MAX_HEIGHT)
	//    dim.height=DialogFactory.MAX_HEIGHT;
	//setSize(dim);
	//textPane.getPreferredSize() always returns 0,0!
	setSize(DialogFactory.MAX_WIDTH,DialogFactory.MAX_HEIGHT);
	//System.out.println(content);
	Bundle.setLocation(this);
    }

    /**
     * get anchor style, with property (address) set
     */
    private static Style getAnchorStyle(StyleContext sc,String address) {
	Style anchorS=sc.addStyle(null,null);
	StyleConstants.setItalic(anchorS,true);
	StyleConstants.setForeground(anchorS,Color.blue);
	StyleConstants.setUnderline(anchorS,true);
	anchorS.addAttributes(sc.addAttribute(anchorS,ANCHOR,address));
	return anchorS;
    }

    /**
     * initialize the document itself
     */
    private static void initDoc(Document doc,StyleContext sc,String content) {
	//symbol style
	Style symbolS=sc.addStyle(null,null);
	StyleConstants.setForeground(symbolS,SYMBOL_COLOR);
	StyleConstants.setBold(symbolS,true);

	//simple type style
	Style typeS=sc.addStyle(null,null);
	StyleConstants.setForeground(typeS,TYPE_COLOR);
	StyleConstants.setBold(typeS,true);

	//key words
	Style keyS=sc.addStyle(null,null);
	StyleConstants.setForeground(keyS,KEY_COLOR);
	StyleConstants.setBold(keyS,true);
	
	//loop words
	Style loopS=sc.addStyle(null,null);
	StyleConstants.setForeground(loopS,LOOP_COLOR);
	StyleConstants.setBold(loopS,true);

	//string style
	Style stringS=sc.addStyle(null,null);
	StyleConstants.setForeground(stringS,STRING_COLOR);
	StyleConstants.setBold(stringS,true);

	//comment 1 style
	Style comment1S=sc.addStyle(null,null);
	StyleConstants.setForeground(comment1S,COMMENT1_COLOR);
	StyleConstants.setItalic(comment1S,true);
	
	//comment x style
	Style commentxS=sc.addStyle(null,null);
	StyleConstants.setForeground(commentxS,COMMENTx_COLOR);
	StyleConstants.setItalic(commentxS,true);

	//tags
	Style tagS=sc.addStyle(null,null);
	StyleConstants.setForeground(tagS,COMMENTx_COLOR);
	StyleConstants.setItalic(tagS,true);
	StyleConstants.setBold(tagS,true);

	//modifiers style
	Style modifS=sc.addStyle(null,null);
	StyleConstants.setForeground(modifS,MODIF_COLOR);
	StyleConstants.setBold(modifS,true);

	//public style
	Style publicS=sc.addStyle(null,null);
	StyleConstants.setForeground(publicS,PUBLIC_COLOR);
	StyleConstants.setBold(publicS,true);

	//protected style
	Style protectedS=sc.addStyle(null,null);
	StyleConstants.setForeground(protectedS,PROTECTED_COLOR);
	StyleConstants.setBold(protectedS,true);
	
	//private style
	Style privateS=sc.addStyle(null,null);
	StyleConstants.setForeground(privateS,PRIVATE_COLOR);
	StyleConstants.setBold(privateS,true);

	//throw style
	Style throwS=sc.addStyle(null,null);
	StyleConstants.setForeground(throwS,THROW_COLOR);
	StyleConstants.setBold(throwS,true);

	//final/abstract style
	Style finalS=sc.addStyle(null,null);
	StyleConstants.setForeground(finalS,FINAL_COLOR);
	StyleConstants.setBold(finalS,true);

	//bool style (true/false)
	Style boolS=sc.addStyle(null,null);
	StyleConstants.setBold(boolS,true);

	//code style
	Style codeS=sc.addStyle(null,null);
	//StyleConstants.setFontFamily(codeS,"Fixed");

	Style[] styles=new Style[10];
	styles[0]=modifS;
	styles[1]=throwS;
	styles[2]=finalS;
	styles[3]=typeS;
	styles[4]=publicS;
	styles[5]=protectedS;
	styles[6]=privateS;
	styles[7]=keyS;
	styles[8]=loopS;
	styles[9]=boolS;

	int max=content.length();
	char[] txt=new char[max+1];
	content.getChars(0,max,txt,0);
	txt[max]=' ';
	char c1=txt[0],c2;
	int start=0,end=0,stop;
	char id;
	boolean incode=false;

	try {
	loop:
	    while(end<max) {
		switch(c1) {

		case '\'': //char
		case '\"': //string
		    if (start<end)
			doc.insertString(start,new String(txt,start,end-start),symbolS);
		    start=end++;
		    while(end<max && (c2=txt[end])!=c1) {
			if (c2=='\\') end+=2;
			else end++;
		    }
		    if (end<max) {
			c1=txt[++end];
			doc.insertString(start,new String(txt,start,end-start),stringS);
			start=end;
			continue;
		    } else {
			doc.insertString(start, new String(txt,start,max-start),stringS);
			start=end=max;
			break loop;
		    }

		case '/': // / or /* or //
		    switch(txt[end+1]) {
		    case '/': //comment // comment1S
			if (start<end)
			    doc.insertString(start,new String(txt,start,end-start),symbolS);
		        start=end; end+=2;
			while(end<max && txt[end++]!='\n');
			if (end<max) {
			    doc.insertString(start,new String(txt,start,end-start),comment1S);
			    c1=txt[start=end];
			    continue;
			} else {
			    doc.insertString(start,new String(txt,start,max-start),comment1S);
			    start=end=max;
			    break loop;
			}
		    case '*': //comment /* commentxS
			if (start<end)
			    doc.insertString(start,new String(txt,start,end-start),symbolS);
			start=end; end+=2;
			if (end+2>=max) break loop;
			c1=txt[end++];
			while(end<max && ((c2=txt[end++])!='/' || c1!='*')) {
			    switch(c1) {
			    case '@':
				switch(c2) {
				case 's':
				    if (end+2<max &&
					(txt[end]^(txt[end+1]<<8))==('e'^('e'<<8)) &&
					(txt[end+2]==' ' || txt[end+2]=='\t')) {
					doc.insertString(start,new String(txt,start,end-start-2),commentxS);
					start=end-2;
					end+=3;
					while(end<max && (txt[end]==' ' || txt[end]=='\t')) end++;
					doc.insertString(start,new String(txt,start,end-start),tagS);
					start=end;
				    address:
					while(end<max) {
					    switch(txt[end++]) {
					    case '\n':
					    case ' ':
					    case '\t':
						break address;
					    }
					}
					if (end-start>1) {
					    String addr=new String(txt,start,end-start-1);
					    doc.insertString(start,addr,getAnchorStyle(sc,addr));
					    start=end-1;
					}
					c1=txt[end++];
					continue;
				    } else if (end+4<max && recognize(txt,end,_SINCE,4)) {
					doc.insertString(start,new String(txt,start,end-start-2),commentxS);
					doc.insertString(end-2,"@since",tagS);
					start=(end+=4);
					c1=' ';
					continue;
				    }
				    c1=c2;
				    continue;
				case 'p':
				    if (end+4<max && recognize(txt,end,_PARAM,4)) {
					doc.insertString(start,new String(txt,start,end-start-2),commentxS);
					doc.insertString(end-2,"@param",tagS);
					start=(end+=4);
					c1=' ';
					continue;
				    }
				    c1=c2;
				    continue;
				case 'r':
				    if (end+5<max && recognize(txt,end,_RETURN,5)) {
					doc.insertString(start,new String(txt,start,end-start-2),commentxS);
					doc.insertString(end-2,"@return",tagS);
					start=(end+=5);
					c1=' ';
					continue;
				    }
				    c1=c2;
				    continue;
				case 'e':
				    if (end+8<max && recognize(txt,end,_EXCEPTION,8)) {
					doc.insertString(start,new String(txt,start,end-start-2),commentxS);
					doc.insertString(end-2,"@exception",tagS);
					start=(end+=8);
					c1=' ';
					continue;
				    }
				    c1=c2;
				    continue;
				case 'd':
				    if (end+9<max && recognize(txt,end,_DEPRECATED,9)) {
					doc.insertString(start,new String(txt,start,end-start-2),commentxS);
					doc.insertString(end-2,"@deprecated",tagS);
					start=(end+=9);
					c1=' ';
					continue;
				    }
				    c1=c2;
				    continue;
				}
			    case '*':
				if (incode)
				    txt[end-2]=' ';
				c1=c2;
				continue;
			    case '<':
				switch(c2) {
				case 'p':
				    if (end<max && txt[end]=='>') {
					erase(txt,end-2,3);
					end++;
					c1=' ';
					continue;
				    } else if (end+3<max && recognize1(txt,end,_PRE,2)) {
					erase(txt,end-2,5);
					incode=true;
					end+=3;
					doc.insertString(start,new String(txt,start,end-start),commentxS);
					start=end;
					c1=' ';
					continue;
				    }
				    c1=c2;
				    continue;
				case 'c':
				    if (end+4<max && recognize1(txt,end,_CODE,3)) {
					erase(txt,end-2,6);
					incode=true;
					end+=4;
					doc.insertString(start,new String(txt,start,end-start),commentxS);
					start=end;
					c1=' ';
					continue;
				    }
				    c1=c2;
				    continue;
				case 'l':
				    if (end+1<max && (txt[end]^(txt[end+1]<<8))==('i'^('>'<<8))) {
					doc.insertString(start,new String(txt,start,end-start-2),commentxS);
					doc.insertString(end-2,"  \u2778 ",tagS);
					start=(end+=2);
					c1='>';
					continue;
				    }
				    c1=c2;
				    continue;
				case '/':
				    if (end+11<max) {
					if (recognize0(txt,end,_PRE,3)) {
					    erase(txt,end-2,6);
					    end+=4;
					    incode=false;
					    doc.insertString(start,new String(txt,start,end-start),codeS);
					    start=end;
					    c1=' ';
					    continue;
					} else if (recognize0(txt,end,_CODE,4)) {
					    erase(txt,end-2,7);
					    end+=5;
					    incode=false;
					    doc.insertString(start,new String(txt,start,end-start),codeS);
					    start=end;
					    c1=' ';
					    continue;
					}
				    }
				default:
				    if (c2=='/' || (c2>='a' && c2<='z')) {
					stop=end-2;
					while(end<max && (txt[end]>='a' && txt[end]<='z')) end++;
					if (txt[end]=='>') {
					    erase(txt,stop,++end-stop);
					    c1=' ';
					    continue;
					}
				    }
				}
			    case '&':
				switch(c2) {
				case 'a':
				    if (end+2<max && (txt[end]^(txt[end+1]<<8)^(txt[end+2]<<16))==('m'^('p'<<8)^(';'<<16))) {
					erase(txt,end-2,5);
					txt[end]='&';
					c1=' ';
					continue;
				    }
				    break;
				case 'l':
				    if (end+1<max && (txt[end]^(txt[end+1]<<8))==('t'^(';'<<8))) {
					erase(txt,end-2,4);
					txt[end]='<';
					c1=' ';
					continue;
				    }
				}
				if (c2>='a'&&c2<='z') {
				    stop=end-2;
				    while(end<max && (txt[end]>='a' && txt[end]<='z')) end++;
				    if (txt[end]==';')
					erase(txt,stop,++end-stop);
				    c1=' ';
				    continue;
				}
			    }
			    c1=c2;
			}
			if (end<max) {
			    doc.insertString(start,new String(txt,start,end-start),commentxS);
			    c1=txt[start=end];
			    continue;
			} else {
			    doc.insertString(start,new String(txt,start,max-start),commentxS);
			    start=end=max;
			    break loop;
			}
		    default:
			c1=txt[++end];
			continue;
		    }

		}

		if (!Character.isJavaIdentifierPart(c1)) {
		    c1=txt[++end];
		    continue;
		}

		if (start<end) {
		    doc.insertString(start,new String(txt,start,end-start),symbolS);
		    start=end;
		}
		
		if (c1>='a' && c1<='w' && (id=recognize(txt,end+1,max,KEYID[c1-'a']))!=0) {
		    end+=(id>>4);
		    doc.insertString(start,new String(txt,start,end-start),styles[id&0xF]);
		} else {
		    while(end<max && Character.isJavaIdentifierPart(txt[++end]));
		    doc.insertString(start,new String(txt,start,end-start),
			       (Character.isUpperCase(c1))?boolS:codeS);
		}
		start=end;
		c1=txt[end];
	    }

	    //end => take what is left
	    if (start<end) {
		doc.insertString(start,new String(txt,start,end-start),symbolS);
		start=end;
	    }

	} catch(Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	}
    }

    /**
     * recognize a keyword
     * @return the associated code ID
     */
    private static char recognize(char[] txt,int pos,int max,char[][] def) {
	if (def==null) return 0;
	int j,len,n=def.length,offset=pos;
	char[] string;
    search:
	for(int i=0;i<n;i++) {
	    string=def[i];
	    len=offset+string.length-1;
	    if (len<max) {
		for(pos=offset;pos<len;pos++)
		    if (txt[pos]!=string[pos-offset])
			continue search;
		if (!Character.isJavaIdentifierPart(txt[pos]))
		    return string[string.length-1];
	    }
	}
	return 0;
    }

    /**
     * recognize a simple keyword
     * @return true if recognized
     */
    private static boolean recognize(char[] txt,int pos,char[] def,int n) {
	if (txt[pos+n]!=' ' && txt[pos+n]!='\n')
	    return false;
	for(int i=0;i<n;i++)
	    if (txt[pos+i]!=def[i])
		return false;
	return true;
    }
    private static boolean recognize0(char[] txt,int pos,char[] def,int n) {
	if (txt[pos+n]!='>')
	    return false;
	for(int i=0;i<n;i++)
	    if (txt[pos+i]!=def[i])
		return false;
	return true;
    }
    private static boolean recognize1(char[] txt,int pos,char[] def,int n) {
	if (txt[pos+n]!='>')
	    return false;
	for(int i=0;i<n;i++)
	    if (txt[pos+i]!=def[i+1])
		return false;
	return true;
    }

    /**
     * erase text
     */
    private static void erase(char[] txt,int pos,int n) {
	for(int i=0;i<n;i++) txt[pos+i]=' ';
    }

    /**
     * implements ActionListener
     */
    public void actionPerformed(ActionEvent e) {
	Object source=e.getSource();
	if (source.equals(close)) {
	    setVisible(false);
	    if (multiple) dispose();
	} else if (source.equals(open))
	    showInfo(input.getText().trim());
	else if (source.equals(input) || source.equals(find)) {
	    String findText=input.getText().trim();
	    if (findText.length()>0) {
		try {
		    Document doc=textPane.getDocument();
		    String content=doc.getText(0,doc.getLength());
		    int pos=content.indexOf(findText,textPane.getCaretPosition());
		    if (pos<0 && textPane.getCaretPosition()>0)
			pos=content.indexOf(findText);
		    if (pos>=0)
			textPane.select(pos,pos+findText.length());
		    //System.out.println(pos+","+(pos+findText.length())+":"+content.substring(pos,pos+findText.length()));
		}
		catch(Exception x) {
		    System.err.println(x);
		}
	    }
	} else if (source.equals(back)) {
	    if (where==0) back.setEnabled(false); //correct error if any
	    else {
		where--;
		className=history[where].className;
		setTitle(history[where].title);
		scroller.getViewport().add(textPane=history[where].textPane);
		textPane.dispatchEvent(new ComponentEvent(textPane,ComponentEvent.COMPONENT_RESIZED));
		back.setEnabled((where>0));
		next.setEnabled(true);
	    }
	} else if (source.equals(next)) {
	    if (where==size) next.setEnabled(false); //correct error if any
	    else {
		where++;
		className=history[where].className;
		setTitle(history[where].title);
		scroller.getViewport().add(textPane=history[where].textPane);
		textPane.dispatchEvent(new ComponentEvent(textPane,ComponentEvent.COMPONENT_RESIZED));
		back.setEnabled(true);
		next.setEnabled((where<size));
	    }
	}
    }

    /**
     * implements HyperlinkListener
     *
    public void hyperlinkUpdate(HyperlinkEvent e) {
	HyperlinkEvent.EventType event=e.getEventType();
	if (event.equals(event.ACTIVATED))
	    showInfo(e.getDescription().trim());
	else if (event.equals(event.ENTERED))
	    edit.setCursor(NiceCheckbox.HAND);
	else if (event.equals(event.EXITED))
	    edit.setCursor(defaultCursor);
    }
    */

    /**
     * show information from a tag/ref basis
     */
    public void showInfo(String theDescription) {
	String description=theDescription;
	//set the default package
	if (!description.startsWith("#") && description.indexOf('.')<0) {
	    int index=className.lastIndexOf('.');
	    if (index>0)
		description=className.substring(0,index+1)+description;
	}
	//System.out.println("["+className+"-"+description+"]");
	try { //check validity of the name
	    Class.forName(description);
	} catch(Throwable hopsan) {
	    Class[] maybe=ClassReflect.findClasses(theDescription,false);
	    if (maybe.length>0)
		description=maybe[0].getName();
	    //else fail anyway
	}
	String info=getInfoOn(description);
	show(className,
	     description.startsWith("#")?className+description:description,
	     (info==null)?"null":info);
    } 
    
    /**
     * @return the extracted class name or the default one if not found
     */
    public static String getClassName(String extractHere) {
	int index=extractHere.indexOf('#');
	if (index<=0)
	    return extractHere;
	else
	    return extractHere.substring(0,index);
    }

    /**
     * show a new content
     */
    public void show(String h_className,String h_title,String content) {
	String key=h_className+"&"+h_title+master.getSig();
	//System.out.println(h_className);
	textPane=(JTextPane)cache.get(key);
	if (textPane==null) {
	    StyleContext context=new StyleContext();
	    DefaultStyledDocument h_doc=new DefaultStyledDocument(context);
	    initDoc(h_doc,context,content);
	    textPane=new JTextPane(h_doc);
	    textPane.setEditable(false);
	    textPane.setSelectionColor(Color.darkGray);
	    textPane.addMouseMotionListener(this);
	    textPane.addMouseListener(this);
	    cache.put(key,textPane);
	    key=null;
	}
	if (where>0 || isVisible())
	    where++;
	size=where;
	if (where>=history.length) {
	    HistoryEntry[] larger=new HistoryEntry[history.length+10];
	    System.arraycopy(history,0,larger,0,where-1);
	    history=larger;
	}
	if (history[where]==null)
	    history[where]=new HistoryEntry(h_className,h_title,textPane);
	else {
	    history[where].className=h_className;
	    history[where].title=h_title;
	    history[where].textPane=textPane;
	}
	next.setEnabled(false);
	back.setEnabled((where>0));
	setTitle(h_title);
	className=h_className;
	textPane.setCaretPosition(0);
	scroller.getViewport().add(textPane);
	setVisible(true);
	if (key!=null)
	    textPane.dispatchEvent(new ComponentEvent(textPane,ComponentEvent.COMPONENT_RESIZED));
	textPane.revalidate();
	textPane.repaint();
    }

    /**
     * basically shows the window...
     */
    public static void show(String className,String title,String content,Assistant master) {
	new ViewerThread(className,title,content,master).run();
    }

    /**
     * @show information on the requested infoname:
     * @param infoname full.class.name#a_method name or
     * just full.class.name
     */
    public String getInfoOn(String infoname) {
	//System.out.println(className+"> "+infoname);
	int index=infoname.indexOf('#');
	String classname,methodname,arglist[]=null;
	if (index<0) {
	    classname=infoname;
	    methodname=null;
	} else {
	    if (index==0)
		classname=className;
	    else
		classname=infoname.substring(0,index);
	    if (index+1<infoname.length()) {
		int index1=infoname.indexOf('(',index+1);
		if (index1>0) {
		    methodname=infoname.substring(index+1,index1);
		    int index2=infoname.indexOf(')',index1+1);
		    if (index2<0) index2=infoname.length();
		    StringTokenizer tokenizer=new StringTokenizer(infoname.substring(index1+1,index2)," \t,");
		    Vector args=new Vector();
		    while(tokenizer.hasMoreTokens()) {
			String arg=tokenizer.nextToken();
			if (arg.indexOf('.')<0) {
			    //short name->find full one!
			    Class[] completion=ClassReflect.findClasses(arg,false);
			    if (completion.length>0)
				arg=completion[0].getName();
			}
			args.addElement(arg);
		    }
		    arglist=new String[args.size()];
		    args.copyInto(arglist);
		} else
		    methodname=infoname.substring(index+1);
	    } else
		methodname=null;
	}
	
	Class[] result=ClassReflect.findClasses(classname,false);
	if (result.length>0) {
	    Class theclass=result[0];
	    String source=master.getSource(theclass);
	    String extract=null;
	    className=theclass.getName();
	    if (methodname!=null && source!=null) {
		extract=JavaFinder.getSourcePos(methodname,
						(arglist==null)?new String[0]:arglist,
						source).substring(source);

		if (extract!=null) return extract;
		
		try {
		    Method[] methods=theclass.getDeclaredMethods();
		    for(int i=0;i<methods.length;i++)
			if (methods[i].getName().equals(methodname)) {
			    extract=SortableMethod.getSource(source,methods[i]);
			    if (extract!=null) return extract;
			}
		} catch(Exception e) {
		    System.err.println(e);
		}
		
		try {
		    Constructor[] constructors=theclass.getDeclaredConstructors();
		    for(int i=0;i<constructors.length;i++)
			if (constructors[i].getName().equals(methodname)) {
			    extract=SortableConstructor.getSource(source,constructors[i]);
			    if (extract!=null) return extract;
			}
		} catch(Exception e) {
		    System.err.println(e);
		}
	    }
	    return source;
	}

	return null;
    }

    /**
     * Invoked when the mouse has been clicked on a component.
     */
    public void mouseClicked(MouseEvent e) {
	Object anchor=getAnchor(e);
	if (anchor!=null) {
	    showInfo(anchor.toString());
	    //System.out.println("click("+anchor+")");
	}
    }

    /**
     * useless methods from listeners
     */
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseDragged(MouseEvent e) {}

    /**
     * Invoked when the mouse button has been moved on a component
     * (with no buttons no down).
     */
    public void mouseMoved(MouseEvent e) {
	getAnchor(e);
    }

    /**
     * @return the (perhaps) anchor the cursor is on
     * and manage the cursor itself
     */
    private Object getAnchor(MouseEvent e) {
	JTextPane text=(JTextPane)e.getSource();
	int pos=text.viewToModel(new Point(e.getX(),e.getY()));
	if (pos>=0) {
	    Element elem=((DefaultStyledDocument)text.getDocument()).getCharacterElement(pos);
	    if (curElem!=elem) {
		curElem=elem;
		Object ref=elem.getAttributes().getAttribute(ANCHOR);
		if (anchorRef!=ref) {
		    if (anchorRef==null) {
			//System.out.println("in:"+ref);
			textPane.setCursor(HANDCURSOR);
		    }
		    else if (ref==null) {
			//System.out.println("out:"+anchorRef);
			textPane.setCursor(defaultCursor);
		    }
		    anchorRef=ref;
		}
		return ref;
	    }
	    return anchorRef;
	}
	return null;
    }

    /**
     * initialize caret position
     */
    void initPos() {
	textPane.setCaretPosition(0);
    }
    
}


/**
 * a thread to run in parallel all the expensive stuff
 */
class ViewerThread extends Thread {
    String classname,title,content;
    Assistant master;
    ViewerThread(String classname,String title,String content,Assistant master) {
	this.classname=classname;
	this.title=title;
	this.content=content;
	this.master=master;
    }
    //run() does the job
    public void run() {
	JViewer viewer=new JViewer(classname,title,content,master);
	//try{java.io.FileWriter w=new java.io.FileWriter("Test.html");w.write(content);w.close();}
	//catch(Exception e){}
	//Bundle.show(viewer);
	viewer.setVisible(true);
	//put this here  otherwise java hangs up (IBM JDK), do not ask me why!
	viewer.initPos();
	//System.out.println(content);
    }
}

/**
 * entry for history
 */
class HistoryEntry {
    String className,title;
    //DefaultStyledDocument doc;
    JTextPane textPane;
    HistoryEntry(String className,String title,JTextPane textPane) {
	this.className=className;
	this.title=title;
	//this.doc=doc;
	this.textPane=textPane;
    }
}







