/**
 * RadioButtonMenuItem
 * A radio button menu item linked to a toggle button
 * to keep states coherent.
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.swing;

import javax.swing.JRadioButtonMenuItem;
import javax.swing.Icon;
import javax.swing.AbstractButton;
import java.awt.event.*;


public class RadioButtonMenuItem extends JRadioButtonMenuItem implements ActionListener {

  /**
   * the peer button, to be updated (selected)
   */
  AbstractButton button;
  
  /**
   * contructors: just register the peer button
   */
  public RadioButtonMenuItem(AbstractButton button) {
    this(null,null,button);
  }
  public RadioButtonMenuItem(Icon icon,AbstractButton button) {
    this(null,icon,button);
  }
  public RadioButtonMenuItem(String name,AbstractButton button) {
    this(name,null,button);
  }
  public RadioButtonMenuItem(String name,Icon icon,AbstractButton button) {
    super(name,icon);
    this.button=button;
    addActionListener(this);
    if (button!=null)
      button.addActionListener(this);
  }
  
  /**
   * keep the peer button in the same state
   */
  public void setEnabled(boolean b) {
    super.setEnabled(b);
    if (button!=null && (b!=button.isEnabled()))
      button.setEnabled(b);
  }

  /**
   * keep the peer button in the same state
   */
  public void setSelected(boolean b) {
    super.setSelected(b);
    if (button!=null && (b!=button.isSelected()))
      button.setSelected(b);
  }

  /**
   * returns the associated button
   */
  public AbstractButton getButton() {
    return button;
  }

  /**
   * changes the associated button
   */
  public void setButton(AbstractButton button) {
    if (this.button!=null)
      this.button.removeActionListener(this);
    this.button=button;
    if (button!=null)
      button.addActionListener(this);
    button.setEnabled(isEnabled());
    button.setSelected(isSelected());
  }

  /**
   * keeps peers in the same states
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(this))
      button.setSelected(isSelected());
    else if (e.getSource().equals(button))
      super.setSelected(button.isSelected());
  }

}
