/**
 * RaiseableBorder
 * A listener which raise/lower borders of buttons when
 * pressed/released
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.0: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.swing;

import javax.swing.border.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;


public class RaiseableBorder implements MouseListener,ChangeListener,MouseMotionListener {
  
  /**
   * the only instance of this class
   */
  private static RaiseableBorder instance=new RaiseableBorder();

  /**
   * add a button to be monitored
   */
  public static void add(AbstractButton button) {
    button.setBorder(BorderFactory.createRaisedBevelBorder());
    button.addMouseListener(instance);
    button.addMouseMotionListener(instance);
    button.addChangeListener(instance);
  }
  
  /**
   * remove a button (remove the instance from the button's listener list)
   */
  public static void remove(AbstractButton button) {
    button.removeMouseListener(instance);
    button.removeMouseMotionListener(instance);
    button.removeChangeListener(instance);
  }

  /**
   * lower the border if enabled and raised border
   */
  public void mousePressed(MouseEvent e) {
    AbstractButton button=(AbstractButton)e.getSource();
    if (button.isEnabled()) {
      Border border=button.getBorder();
      if (border instanceof BevelBorder
	  && ((BevelBorder)border).getBevelType()==BevelBorder.RAISED)
	button.setBorder(BorderFactory.createLoweredBevelBorder());
    }
  }

  /**
   * raise the border if enabled and lowered border
   */
  public void mouseReleased(MouseEvent e) {
    AbstractButton button=(AbstractButton)e.getSource();
    if (button.isEnabled() && !button.isSelected()) {
      Border border=button.getBorder();
      if (!button.isSelected() && border instanceof BevelBorder
	  && ((BevelBorder)border).getBevelType()==BevelBorder.LOWERED)
	button.setBorder(BorderFactory.createRaisedBevelBorder());
    }
  }

  /**
   * update the look if the state changes
   */
  public void stateChanged(ChangeEvent e) {
    AbstractButton button=(AbstractButton)e.getSource();
    Border border=button.getBorder();
    if (border instanceof BevelBorder) {
      BevelBorder bb=(BevelBorder)border;
      if (!button.isSelected() && bb.getBevelType()==BevelBorder.LOWERED)
	button.setBorder(BorderFactory.createRaisedBevelBorder());
      else if (button.isSelected() && bb.getBevelType()==BevelBorder.RAISED)
	button.setBorder(BorderFactory.createLoweredBevelBorder());
    }
  }

  /**
   * to correct when the mouse is dragged out/in
   */
  public void mouseDragged(MouseEvent e) {
    mousePressed(e);
  }

  /**
   * useless methods of MouseListener
   */
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mouseClicked(MouseEvent e) {}
  public void mouseMoved(MouseEvent e) {}
}
