/**
 * ToggleButtonMenuItem
 * a tobble button linked to a check box menu item
 * so that the states are kept coherent
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.0: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.swing;

import javax.swing.JMenuItem;
import javax.swing.Icon;
import javax.swing.AbstractButton;
import javax.swing.event.*;


public class ToggleButtonMenuItem extends ButtonMenuItem implements ChangeListener {

  Icon normalIcon,disabledIcon;

  /**
   * contructors: just register the peer button
   */
  public ToggleButtonMenuItem(AbstractButton button) {
    this(null,null,button);
  }
  public ToggleButtonMenuItem(Icon icon,AbstractButton button) {
    this(null,icon,button);
  }
  public ToggleButtonMenuItem(String name,AbstractButton button) {
    this(name,null,button);
  }
  public ToggleButtonMenuItem(String name,Icon icon,AbstractButton button) {
    super(name,icon,button);
    if (button!=null)
      button.addChangeListener(this);
    normalIcon=icon;
  }  

  /**
   * manage the icon ourselves
   */
  public void setIcon(Icon icon) {
    normalIcon=icon;
    super.setIcon(icon);
  }
  public void setDisabledIcon(Icon icon) {
    disabledIcon=icon;
    super.setDisabledIcon(icon);
  }

  /**
   * changes the associated button
   */
  public void setButton(AbstractButton button) {
    if (this.button!=null)
      this.button.removeChangeListener(this);
    if (button!=null)
      button.addChangeListener(this);
    super.setButton(button);
  }

  /**
   * change listener
   */
  public void stateChanged(ChangeEvent e) {
    if (button!=null) {
      if (button.isSelected()) {
	if (isEnabled())
	  super.setIcon(getSelectedIcon());
	else if (getDisabledSelectedIcon()!=null)
	  super.setDisabledIcon(getDisabledSelectedIcon());
      }  
      else {
	if (isEnabled())
	  super.setIcon(normalIcon);
	else if (disabledIcon!=null)
	  super.setIcon(disabledIcon);
      }
    }
  }

}
