/**
 * Assistant main program
 * @author Alexandre David
 * @version 3.4
 * bug: desactivating homonyms and all matches
 * does not restore the default list contents
 */

/**
 * Changes history:
 * 3.0: complete rewrite of Assistant 2.4
 * 3.0.1 990721: change a bit the license text,
 *       add some comments and clean a bit the code.
 * 3.0.2 990731: add constant oneclassstr
 * 3.0.3 990803: fixed a known but forgotten bug
 *               concerning updates when the checkbox
 *               related to match options were unselected,
 *               now the class list is updated.
 * 3.0.4 990829: add option -update
 * 3.1 990905: add choice of decompiler, support
 *             for future decompilers, all by reflection!
 * 3.2 990905: added decompiler options
 * 3.3 991010: added html option to view the source code
 *    + forgotten license terms that were on the web page only
 * 3.3.1 991104: small fix concerning the entered string:
 *    no search if empty, begins or ends with, changed license to LGPL
 * 3.4 991204: support for multiple/single window for JViewer
 *             bug in getSource fixed, checkSource simplified
 *             swing 1.1.1 window blit option
 */

package adavid.util;

import adavid.reflect.*;
import adavid.awt.*;
import java.awt.*;
import java.awt.event.*;
import adavid.io.*;
import java.io.*;
import java.lang.reflect.Modifier;
import java.util.Vector;
import java.util.StringTokenizer;
import adavid.swing.JViewer;

/**
 * Assistant 3.x : redesigned Assistant from version 2.4
 */
public class Assistant extends Frame implements ItemListener,ActionListener {

  /**
   * version and final constants
   */
  public static final String VERSION="Assistant 3.4",AUTHOR="\n\nAlexandre DAVID";
  public static final String NOTHING="Nothing",INFO="Info",BUSY="Busy...";
  public static final String CONSTRUCT="Constructors",FIELDS="Fields",METHODS="Methods";

    /**
     * available decompilers
     */
    Decompiler[] decompiler;
    //current used
    int decompilerID=0;

  /**
   * the path of the default icons, so that Bundle finds them
   * this can be overriden in a simple Assistant.properties file
   */
  public static String path$icons="adavid/icons/";

    /**
     * the list of decompilers, please use Assistant.properties to change it
     * resource name: decompilerList.name
     */
    public static String decompilerList="mocha.MochaGate";

  /**
   * the icons to be drawn: initialized by Bundle
   */
  public static Image FileI,FileISelected,FileA,FileASelected;

    /**
     * control WindowBlit feature of Swing 1.1.1
     */
    public static Boolean WBLIT=Boolean.TRUE;

   /**
    * messages that can be redefined in a properties file and
    * set by Bundle
    */
  public static String 
      confirmQuitMsg="Are you sure to want to quit?",
      quitMsg="Quit Assistant",
      yesMsg="Yes",
      noMsg="No",
      classInput="Class/Interface:",
      source="Source",
      options="Options",
      about="About",
      constructLabel=CONSTRUCT,
      fieldsLabel=FIELDS,
      methodsLabel=METHODS,
      superClasses="Super classes:",
      interfaces="Interfaces:",
      searchFor="Looking for sources of ",
      forcedstr="All matches",
      notfound="Could not retrieve source!",
      iotitle="stdout/stderr",
      closemsg="Close",
      findmsg="Find",
      anystr="any",
      namestr="Name",
      typestr="Type",
      shortstr="short",
      longstr="long",
      accessstr="Access scope",
      modifstr="Modifiers",
      presentstr="present",
      absentstr="absent",
      javastr="java file",
      jjstr="jj file",
      uncompstr="uncompile",
      disasstr="disassemble",
      sortstr1="activated",
      sortmstr="on modifiers",
      sortstr2="Sorting",
      printstr="print",
      updatePack="Update packages",
      allmatchesstr="Homonyms",
      matchesstr="Class matches:",
      miscstr="Misc",
      packstr="Packages",
      quitstr="Quit",
      packstr2=" packages",
      classstr=" classes",
      oneclassstr=" class",
      pleasestr="Enter a class/interface name please...",
      searchstr="Searching for matches...",
      invokepackviewer="Invoking PackageViewer...",
      lookstr="Looking for ",
      decompilerstr="Decompiler",
      verbosestr="verbose",
      debugstr="debug",
      openstr="Open",
      onehtmlstr="Single Window";

  /**
   * this can be customized in a properties file as well
   */
  public static Color infoColor=Color.blue;

   /**
    * default help text, displayed when clicking the about
    * button
    */
  public static String help=
"This is an assistant for Java programmers. You can find classes, explore sources, uncompile\n"+
"sources (provided that you have the Java decompiler Mocha). The main point of this program is\n"+
"to have a quick access to the available methods, fields and constructors of any accessible\n"+
"class on your system. The input field Class/Interfaces is to enter a short or long name of a\n"+
"class or an interface. When you press enter, the program tries to find the class/interface.\n"+
"The source button gives access to the source. The option button controls the behaviour of the\n"+
"program: filter the names, the access rights, how to search classes (stop to the first found or\n"+
"search all, which is slower)... The about button displays this help text.\n"+
"\n"+
"When a class is found, then you have to choose to visualise constructors, fields or methods of\n"+
"that class. The names will be filtered as precised in the options and the names are sorted in\n"+
"alphabetical order. By clicking on a constructor or a method gives you the extracted portion\n"+
"of code which contains this constructor or method.\n"+
"\n"+
"Configuration of the program:\n"+
"* the program uses your CLASSPATH environment variable since Java finds its classes with this\n"+
"  information.\n"+
"* you can specify a path where to look for the sources with an option when starting the program:\n"+
"  java -Djava.source.path=my_path/:a_path/zip_file.zip:file.zip[sub_dir] adavid.util.Assistant\n"+
"  By this way you can tell the program to look at directories and zip files, and sub directories\n"+
"  included in zip files.\n"+
"* you can configure your own language if you run:\n"+
"  java adavid.util.JDefaultResource adavid.util.Assistant\n"+
"  which gives you a file with default definitions that you can change. Next time you start the\n"+
"  program, you get the new interface.\n"+
"  Other options are customizable, see the produced file, and\n"+
"  VERY important: you can add here a list of decompilers if you want!\n"+
"  The rule is: the list is space separated and the names are full class names\n"+
"  of a classes which implement adavid.reflect.Decompiler\n"+
"  mocha.MochaGate is however always included.\n"+
"* you can redirect stdout/stderr in a window which opens when something is written by giving the\n"+
"  option -redirect when invoking Assistant.\n"+
"* you can enable the confirm dialog when exiting by passing the option -confirm\n"+
"* you can disable the window blit feature of Swing 1.1.1 if you have problems with\n"+
"  the option -nowindowblit. This option accelerates the scrolling in Swing.\n"+
"* sometimes you may experience problems with the saved classpath file in $HOME/.PackageFinder.info\n"+
"  so use the option -update to force the update of this file.\n"+
"\n"+
"The virtual machine Kaffe 1.2 works quite bad with Assistant, specially when you select the\n"+
"modifier filters and when you open the package viewer window.\n"+
"\n"+
"License: LGPL see license.html included in the distribution\n"+
"Java Assistant is the property of the author Alexandre David, it is also\n"+
"available from the freashmeat database, and registered by this way.\n";

  /**
   * this is to redefine the usual System.out
   * so that messages get caught and possibly
   * displayed
   */
  StringOutputStream output=new StringOutputStream();

    /**
     * this is to avoid starting many viewers
     */
    JViewer jviewer;

  /**
   * graphical components
   */
  Checkbox checkConstruct,checkFields,checkMethods,allmatches,forced;
  NiceButton sourceButton,optionsButton,packageButton,aboutButton,quitButton;
  TextField input;
  List superList,interfList,constructList,fieldsList,methodsList;
  CardLayout genCards,modeCards;
  Panel modepanel,center;
  NiceLabel statusLabel,superLabel; //an improved label


  /**
   * Assistant variables. currentSuper
   * and currentInterf store the current
   * super classes and interfaces of the
   * current class (currentClass) being
   * examined. The associated constructors,
   * fields and methods wrapped in a sortable
   * class. The source may be null if not
   * yet requested.
   */
  Class[] currentSuper,currentInterf;
  SortableConstructor[] currentConstruct;
  SortableField[] currentFields;
  SortableMethod[] currentMethods;
  String currentSource;
  Class currentClass;
  OptionDialog optionsDialog;
  int sourceOrder=0;
  boolean confirmFlag=false;

  /**
   * one Assistant per JVM,
   * initialized at the beginning,
   * the main function just open it
   */
  private static Assistant assistant;

  /**
   * constructor: protected in case somebody wants to
   * extend it
   */
  protected Assistant() {
    super(VERSION);
    //I want to keep things simple:
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    //basic layout stuff
    setLayout(new BorderLayout());
    setBackground(Color.lightGray);
    setSize(530,350);

    //The initialization of the variables takes
    //place here.
    Bundle bundle=new Bundle(getClass());
    bundle.init(this,this);
    bundle.show(this); //must do so otherwise getGraphics() returns null
    decompiler=findDecompilers();

    //I need a Graphics object in order to build
    //my components.
    Graphics g=getGraphics();
    if (g==null) {
      System.err.println("Panic: getGraphics()==null");
      System.exit(1);
    }

    jviewer=new JViewer("","","",this);

    NiceLabel classLabel=new NiceLabel(g,classInput);
    optionsDialog=new OptionDialog(this);

    //one thing concerning the panel: it has been shown
    //that in practice when adding many items to a list,
    //the list is updated for each item, which results
    //in a loss of performance. This can be easily bypassed
    //by hiding the panel containing the list, which is
    //achieved by a card layout.

    /******** make the north main panel **************/
    Panel northEast=new Panel(new FlowLayout(FlowLayout.CENTER,1,1));
    northEast.add(packageButton=new NiceButton(g,packstr));
    northEast.add(aboutButton=new NiceButton(g,about));
    northEast.add(quitButton=new NiceButton(g,quitstr),BorderLayout.EAST);
    packageButton.addActionListener(this);
    aboutButton.addActionListener(this);
    quitButton.addActionListener(this);

    Panel north=new Panel(new BorderLayout());
    north.add(classLabel,BorderLayout.WEST);
    north.add(northEast,BorderLayout.EAST);
    north.add(input=new TextField(),BorderLayout.CENTER);
    AutoFocuser.add(input); //so that the focus follows the mouse
    input.addActionListener(this);

    /******** the north main panel is added ***********/
    add(north,BorderLayout.NORTH);

    /******** make the main center label **************/
    center=new Panel(genCards=new CardLayout());
    /******** and add it to the main panel ************/
    add(center,BorderLayout.CENTER);

    /******** first card: nothing *********************/
    Panel centerpanel=new Panel(new BorderLayout());
    //centerpanel.add(new Panel(),BorderLayout.CENTER);
    center.add(NOTHING,centerpanel);

    /******** second card: information ***************/
    centerpanel=new Panel(new BorderLayout());
    center.add(INFO,centerpanel);

    /******** super classes and interfaces **********/
    Panel centernorth=new Panel(new BorderLayout());
    
    Panel col2=new Panel(new GridLayout(1,0));
    Panel subcol=new Panel(new BorderLayout());
    Panel matchpanel=new Panel(new FlowLayout(FlowLayout.LEFT,1,1));
    subcol.add(superLabel=new NiceLabel(g,superClasses),BorderLayout.NORTH);
    subcol.add(superList=new List(5),BorderLayout.SOUTH);
    col2.add(subcol);
    subcol=new Panel(new BorderLayout());
    subcol.add(new NiceLabel(g,interfaces),BorderLayout.NORTH);
    subcol.add(interfList=new List(3),BorderLayout.CENTER);
    matchpanel.add(allmatches=new Checkbox(allmatchesstr));
    matchpanel.add(forced=new Checkbox(forcedstr));
    allmatches.addItemListener(this);
    forced.addItemListener(this);
    subcol.add(matchpanel,BorderLayout.SOUTH);
    col2.add(subcol);
    superList.addItemListener(this);
    interfList.addItemListener(this);
    centernorth.add(col2,BorderLayout.NORTH);

    /******* modes: construct, fields, methods *******/
    modepanel=new Panel(new FlowLayout(FlowLayout.LEFT,2,1));
    CheckboxGroup group=new CheckboxGroup();
    modepanel.add(checkConstruct=new Checkbox(constructLabel));
    modepanel.add(checkFields=new Checkbox(fieldsLabel));
    modepanel.add(checkMethods=new Checkbox(methodsLabel));
    checkConstruct.setCheckboxGroup(group);
    checkFields.setCheckboxGroup(group);
    checkMethods.setCheckboxGroup(group);
    checkMethods.setState(true);
    checkConstruct.addItemListener(this);
    checkFields.addItemListener(this);
    checkMethods.addItemListener(this);

    Panel middlebar=new Panel(new BorderLayout());
    middlebar.add(modepanel,BorderLayout.CENTER);
    modepanel=new Panel(new FlowLayout(FlowLayout.RIGHT,1,0));
    modepanel.add(sourceButton=new NiceButton(g,source));
    modepanel.add(optionsButton=new NiceButton(g,options));
    middlebar.add(modepanel,BorderLayout.EAST);
    sourceButton.addActionListener(this);
    sourceButton.setEnabled(false);
    optionsButton.addActionListener(this);
    centernorth.add(middlebar,BorderLayout.SOUTH);

    /******* in the north of the main center panel ******/
    centerpanel.add(centernorth,BorderLayout.NORTH);

    /******** reuse modepanel for the real stuff ********/
    modepanel=new Panel(modeCards=new CardLayout());
    modepanel.add(BUSY,new Panel());
    modepanel.add(CONSTRUCT,constructList=new List(200));
    modepanel.add(FIELDS,fieldsList=new List(200));
    modepanel.add(METHODS,methodsList=new List(200));
    constructList.addItemListener(this);
    methodsList.addItemListener(this);
    centerpanel.add(modepanel,BorderLayout.CENTER);

    NiceCheckbox.makeNice(g,allmatches);
    NiceCheckbox.makeNice(g,forced);
    NiceCheckbox.makeNice(g,checkConstruct);
    NiceCheckbox.makeNice(g,checkMethods);
    NiceCheckbox.makeNice(g,checkFields);
    NiceCheckbox.makeNice(null,superList);
    NiceCheckbox.makeNice(null,interfList);
    NiceCheckbox.makeNice(null,constructList);
    NiceCheckbox.makeNice(null,fieldsList);
    NiceCheckbox.makeNice(null,methodsList);
    
    /******** add the status label in the south *******/
    /*centerpanel.*/add(statusLabel=new NiceLabel(g,""),BorderLayout.SOUTH);
    statusLabel.setForeground(infoColor);
    statusLabel.setText(pleasestr);

    /******** the options dialog **********************/
    doLayout();
    centerpanel.doLayout();
    centernorth.doLayout();
    north.doLayout();
    northEast.doLayout();

    //no after all, the user does this by clicking the package button.
    //PackageViewer.callViewer();

    //to be sure that the assistant is displayed after the initializations
    repaint();
  }

  /**
   * process window events: just close the window when asked
   * I do not need all the stuff of a WindowListener just for this.
   */
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID()==WindowEvent.WINDOW_CLOSING)
      quitFunction();
  }

  /**
   * item listener:
   * show the right card (layout) when a mode is selected
   */
  public void itemStateChanged(ItemEvent e) {
      try {
        Object source=e.getSource();

	if (e.getStateChange()==e.SELECTED) {
	    //I know, this is not nice, but switch does not work here.
	    if (source.equals(checkConstruct)) //the constructor checkbox
		modeCards.show(modepanel,CONSTRUCT);
	    else if (source.equals(checkFields)) //the field checkbox
		modeCards.show(modepanel,FIELDS);
	    else if (source.equals(checkMethods)) //the method checkbox
		modeCards.show(modepanel,METHODS);
	    else if (source.equals(superList)) //the list of super classes
		selectClass(currentSuper[superList.getSelectedIndex()],true);
	    else if (source.equals(interfList)) //the list of interfaces
		selectClass(currentInterf[interfList.getSelectedIndex()],false);
	    else if (source.equals(constructList)) //the list of constructors
		findSource(currentConstruct[constructList.getSelectedIndex()]);
	    else if (source.equals(methodsList)) //this list of methods
		findSource(currentMethods[methodsList.getSelectedIndex()]);
	}
	//select or unselect
	if (source.equals(allmatches) || source.equals(forced)) {
	    //search options
	    superLabel.setText((allmatches.getState() || forced.getState())?matchesstr:superClasses);
	    lookAtClass();
	}
      }
      catch(Throwable t) {
        t.printStackTrace();
      }
      checkstreams(); //open the dialog containing the messages if any.
  }

  /**
   * reselect the current class
   */
  public void selectClass() {
    if (currentClass!=null)
      selectClass(currentClass,false);
  }

    /**
     * @return the source order for JViewer
     */
    public int getSig() {
	return (decompilerID<<8)|sourceOrder;
    }

    /**
     * invalidate the source
     */
    public synchronized void invalidateSource() {
	currentSource=null;
    }

  /**
   * select a class
   */
  public void selectClass(Class wclass,boolean updateAll) {
    //choose the right cards
    genCards.show(center,INFO);
    modeCards.show(modepanel,BUSY);

    //update class
    if (!wclass.equals(currentClass)) {
      currentSource=null;
      currentClass=wclass;
      sourceButton.setEnabled(true);
    }
    //update the class name label
    statusLabel.setText(wclass.toString());

    currentConstruct=
      ClassReflect.getSortableConstructors(ClassReflect.getConstructors(wclass));
    currentFields=
      ClassReflect.getSortableFields(ClassReflect.getFields(wclass));
    currentMethods=
      ClassReflect.getSortableMethods(ClassReflect.getMethods(wclass));

    //false when one clicks on the interface list
    if (updateAll) {
      currentInterf=wclass.getInterfaces();
      if (interfList.getItemCount()>0)
	interfList.removeAll();
      for(int i=0;i<currentInterf.length;i++)
	interfList.addItem(ClassReflect.getString(currentInterf[i]));
    }
    //clear lists
    if (constructList.getItemCount()>0)
      constructList.removeAll();
    if (fieldsList.getItemCount()>0)
      fieldsList.removeAll();
    if (methodsList.getItemCount()>0)
      methodsList.removeAll();

    //sort all
    if (ClassReflect.isSortEnabled()) {
      if (currentConstruct.length>1)
	Sorter.sortI(currentConstruct,0,currentConstruct.length-1);
      if (currentFields.length>1)
	Sorter.sortI(currentFields,0,currentFields.length-1);
      if (currentMethods.length>1)
	Sorter.sortI(currentMethods,0,currentMethods.length-1);
    }
    //System.out.println("lists sorted");

    //here is the slowest part of the program!
    //and this comes from List.addItem(String)
    //The list is not shown, so that we gain much
    //in speed but it is still quite slow.
    if (optionsDialog.printcheck.getState()) {
      for(int i=0;i<currentConstruct.length;i++)
	constructList.addItem(currentConstruct[i].getObjectString());
      for(int i=0;i<currentFields.length;i++)
	fieldsList.addItem(currentFields[i].getObjectString());
      for(int i=0;i<currentMethods.length;i++)
	methodsList.addItem(currentMethods[i].getObjectString());
    }
    else {
      for(int i=0;i<currentConstruct.length;i++)
	constructList.addItem(currentConstruct[i].toString());
      for(int i=0;i<currentFields.length;i++)
	fieldsList.addItem(currentFields[i].toString());
      for(int i=0;i<currentMethods.length;i++)
	methodsList.addItem(currentMethods[i].toString());
    }
    //System.out.println("lists full");

    //displays the right card
    if (checkConstruct.getState())
      modeCards.show(modepanel,CONSTRUCT);
    else if (checkFields.getState())
      modeCards.show(modepanel,FIELDS);
    else if (checkMethods.getState())
      modeCards.show(modepanel,METHODS);

  }

  /**
   * gets the source of the asked method
   */
  public void findSource(SortableMethod method) {
      if (checkSource()) {
	  if (optionsDialog.htmlcheck.getState()) {
	      if (optionsDialog.onehtml.getState())
		  jviewer.show(currentClass.getName(),
				currentClass.getName()+":"+
				ClassReflect.getString(method.getMethod()),
				method.getSource(currentSource));
	      else
		  JViewer.show(currentClass.getName(),
			       currentClass.getName()+":"+
			       ClassReflect.getString(method.getMethod()),
			       //JavaFinder.filterToHTML(method.getSource(currentSource)),//removed
			       method.getSource(currentSource),
			       this);
	  } else
	      DialogFactory.showMessage(this,
					currentClass.getName()+":"+
					ClassReflect.getString(method.getMethod()),
					method.getSource(currentSource),
					closemsg,
					findmsg);
      }
  }
  
  /**
   * gets the source of the asked constructor
   */
  public void findSource(SortableConstructor construct) {
      if (checkSource()) {
	  if (optionsDialog.htmlcheck.getState()) {
	      if (optionsDialog.onehtml.getState())
		  jviewer.show(currentClass.getName(),
				currentClass.getName()+":"+
				ClassReflect.getString(construct.getConstructor()),
				construct.getSource(currentSource));
	      else
		  JViewer.show(currentClass.getName(),
			       currentClass.getName()+":"+
			       ClassReflect.getString(construct.getConstructor()),
			       //JavaFinder.filterToHTML(construct.getSource(currentSource)),//removed
			       construct.getSource(currentSource),
			       this);
	  } else
	      DialogFactory.showMessage(this,
					currentClass.getName()+":"+
					ClassReflect.getString(construct.getConstructor()),
					construct.getSource(currentSource),
					closemsg,
					findmsg);
      }
  }

    /**
     * @return the source of the request class if found
     * @param theclass the class to find the source code
     */
    public String getSource(Class theclass) {
	String source=null;
	//applies the priority search to retrieve the source
	//the tests are like this because any of these tries may fail.
	if (sourceOrder<1)
	    source=JavaFinder.getSource(theclass);
	if (sourceOrder<2 && source==null)
	    source=JavaFinder.getjjSource(theclass);
	if (sourceOrder<3 && source==null) {
	    decompiler[decompilerID].setDecompile();
	    source=decompiler[decompilerID].decompileClass(theclass);
	}
	if (source==null) {
	    decompiler[decompilerID].setDisassemble();
	    source=decompiler[decompilerID].decompileClass(theclass);
	}
	return source;
    }

  /**
   * checks the current source
   */
  public boolean checkSource() {
    if (currentClass==null) {
      sourceButton.setEnabled(false);
      statusLabel.setText("");
      return false;
    }
    if (currentSource==null) {
      statusLabel.setText(searchFor+currentClass.getName());
      currentSource=getSource(currentClass);
      statusLabel.setText(currentClass.toString());
      //result of the search for the source:
      sourceButton.setEnabled(currentSource!=null);
      if (currentSource==null)
	  DialogFactory.showMsg(this,currentClass.getName(),notfound,closemsg);
    }

    return (currentSource!=null);
  }

  /**
   * action listener: for the input field
   */
  public void actionPerformed(ActionEvent e) {
    try {
      Object source=e.getSource();
      if (source.equals(input)) //press enter in the input text field
	lookAtClass();
      else if (source.equals(sourceButton) && checkSource()) //source wanted
	  if (optionsDialog.htmlcheck.getState()) {
	      if (optionsDialog.onehtml.getState())
		  jviewer.show(currentClass.getName(),
				currentClass.getName(),
				currentSource);
	      else
		  JViewer.show(currentClass.getName(),
			       currentClass.getName(),
			       //JavaFinder.filterToHTML(currentSource),//removed
			       currentSource,
			       this);
	  } else
	      DialogFactory.showMessage(this,currentClass.getName(),currentSource,closemsg,findmsg);
      else if (source.equals(optionsButton)) //open the options dialog
	optionsDialog.setVisible(true);
      else if (source.equals(packageButton)) //open the package window
	PackageViewer.callViewer();
      else if (source.equals(quitButton)) //bye bye
	quitFunction();
      else if (source.equals(aboutButton)) //the about text
	DialogFactory.showAlert(this,VERSION,help+AUTHOR,closemsg);
    }
    catch(Throwable t) {
      t.printStackTrace();
    }
    checkstreams(); //check if there is any message to display
  }

  /**
   * quit function: ask for confirm before
   */
  public void quitFunction() {
    if (!confirmFlag || DialogFactory.confirmYN(this,quitMsg,confirmQuitMsg,yesMsg,noMsg)) {
      setVisible(false);
      dispose();
      System.exit(0);
    }
  }

  /**
   * sends a class name to the assistant
   * This is not thread safe, it should not be called by
   * concurrent applications.
   */
  public static void queryClass(String name) {
    if (name!=null) {
      if (assistant==null)
	assistant=new Assistant();
      else
	assistant.setVisible(true);
      assistant.input.setText(name);
      assistant.allmatches.setState(false);
      assistant.superLabel.setText(Assistant.superClasses);
      assistant.lookAtClass();
    }
  }

  /**
   * look at a class
   */
  void lookAtClass() {
    String classname=input.getText().trim();

    if (classname.equals("")) return; //empty
    else if (classname.startsWith(".")||
	     classname.endsWith(".")) { //no chance to work
	input.selectAll();
	statusLabel.setText(pleasestr);
	return;
    }

    statusLabel.setText(lookstr+classname);
    genCards.show(center,NOTHING); //hide all

    Class[] result=null;
    if (!forced.getState()) {
      //PackageFinder is able to display different messages
      //if its labels are set to non null values.
      PackageFinder.setLabels(null,statusLabel,null);
      result=ClassReflect.findClasses(classname,allmatches.getState());
      PackageFinder.setLabels(null,null,null);
    }

    if (result==null || result.length==0) //if failed, try to correct
      result=tryCompletion(classname.toLowerCase());

    //if no result and forced is active, then we have to try the normal mode
    if (forced.getState() && result.length==0) {
      PackageFinder.setLabels(null,statusLabel,null);
      result=ClassReflect.findClasses(classname,allmatches.getState());
      PackageFinder.setLabels(null,null,null);
      forced.setState(false);
      superLabel.setText((allmatches.getState())?matchesstr:superClasses);
    }

    //worked!
    if (result.length>0) {
      if (superList.getItemCount()>0)
	superList.removeAll();
      if (allmatches.getState() || forced.getState()) {
	//display all the results in the "super class" list
	currentSuper=result;
	for(int i=0;i<result.length;i++)
	  superList.addItem(ClassReflect.getString(result[i]));
      }
      else {
	//display all the super classes
	currentSuper=ClassReflect.getHierarchy(result[0]);
	for(int i=0;i<currentSuper.length;i++)
	  superList.addItem(ClassReflect.getString(currentSuper[i]));
      }
      superList.select(0);
      selectClass(currentSuper[0],true);
    }
    else {
      input.selectAll(); //so that the user is invited to correct the query
      statusLabel.setText(pleasestr); //invite the user to do so
    }
  }

  /**
   * try completion
   */
  public Class[] tryCompletion(String name) {
    if (PackageViewer.viewer==null) {
      //I need this class to be initialized for completion
      statusLabel.setText(invokepackviewer);
      PackageViewer.callViewer();
    }
    forced.setState(true);
    superLabel.setText(matchesstr);
    statusLabel.setText(searchstr);
    String[] list=PackageViewer.viewer.getPropositions(name);
    Vector result=new Vector(list.length);
    for(int i=0;i<list.length;i++) {
      try {
	Class theclass=Class.forName(list[i]);
	result.addElement(theclass);
      }
      catch(Throwable notfound){}
    }
    Class[] solutions=new Class[result.size()];
    result.copyInto(solutions);
    if (solutions.length>0)
      input.setText(ClassReflect.getString(solutions[0]));
    return solutions;
  }

  /**
   * check the streams: stderr and stdout ; open a window if necessary
   */
  public void checkstreams() {
    if (output.length()>0) {
      DialogFactory.showAlert(this,iotitle,output.toString(),closemsg);
      output.setLength(0);
    }
  }

  /**
   * the main program, pretty simple
   * since all is done in the static
   * initialization
   */
  public static void main(String[] args) {
    //First initialize the assistant
    if (assistant==null)
      assistant=new Assistant();
    else
      assistant.setVisible(true);

    //look at the command line arguments
    CommandSorter sorter=new CommandSorter(args);
    //redirect I/O
    if (sorter.isDefined("-redirect")) {
      System.setOut(new PrintStream(assistant.output));
      System.setErr(new PrintStream(assistant.output));
    }
    if (sorter.isDefined("-update"))
	PackageFinder.updatePackageResources();
    if (sorter.isDefined("-nowindowblit"))
	WBLIT=Boolean.FALSE;
    //confirm option
    assistant.confirmFlag=sorter.isDefined("-confirm");
  }

    /**
     * find the decompilers
     */
    static Decompiler[] findDecompilers() {
	StringTokenizer tokenizer=new StringTokenizer(decompilerList);
	Vector list=new Vector(),names=new Vector();
	while(tokenizer.hasMoreTokens()) {
	    String token=tokenizer.nextToken();
	    //System.out.println(token);
	    if (!names.contains(token)) {
		names.addElement(token);
		try {
		    Decompiler decompClass=(Decompiler)Class.forName(token).newInstance();
		    //System.out.println(token);
		    list.addElement(decompClass);
		} catch(Throwable t) {
		    System.err.println(t);
		    //t.printStackTrace();
		}
	    }
	}
	if (!names.contains("mocha.MochaGate")) list.addElement(new mocha.MochaGate());
	Decompiler[] result=new Decompiler[list.size()];
	list.copyInto(result);
	return result;
    }
}

/*************************************************
 * a local class used only for the assistant,
 * not nested because I don't like nested classes
 * and nested classes are translated into non
 * nested classes in fact, so the result is
 * the same: in this case it is better to
 * do it myself to have something clean
 *************************************************/

class OptionDialog extends Dialog implements ActionListener,ItemListener {

  NiceCheckbox nameShort,nameLong,typeShort,typeLong,
    anyAccess,publicAccess,protectedAccess,packageAccess,privateAccess,
    anyModif,presentModif,absentModif,javaSrc,jjSrc,uncompSrc,disasSrc,
    sortcheck,sortmodif,printcheck,htmlcheck,onehtml;
  Choice modifiers,decompilerChoice;
  NiceButton closeButton,updatePack,aboutButton,decompileOptions;
  Assistant master;

  /**
   * of course the options are for an assistant
   * no test on master is done, if it is null, then NullPointerException...
   */
  OptionDialog(Assistant master) {
    super(master,master.options);
    this.master=master;
    //same stuff than the assistant
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    //this is executed when the assistant is displayed and
    //it is the assistant which calls this constructor. So
    //this cannot be null, otherwise we have a panic message
    //and the assistant does not start.
    Graphics g=master.getGraphics();
    CheckboxGroup group;

    BorderedPanel namepanel=new BorderedPanel(g,master.namestr,master.infoColor);
    group=new CheckboxGroup();
    nameShort=namepanel.addNiceCheckbox(g,master.shortstr,group,this);
    nameLong=namepanel.addNiceCheckbox(g,master.longstr,group,this);
    namepanel.breakColumn();

    BorderedPanel typepanel=new BorderedPanel(g,master.typestr,master.infoColor);
    group=new CheckboxGroup();
    typeShort=typepanel.addNiceCheckbox(g,master.shortstr,group,this);
    typeLong=typepanel.addNiceCheckbox(g,master.longstr,group,this);
    typepanel.breakColumn();

    BorderedPanel accesspanel=new BorderedPanel(g,master.accessstr,master.infoColor);
    group=new CheckboxGroup();
    anyAccess=accesspanel.addNiceCheckbox(g,master.anystr,group,this);
    publicAccess=accesspanel.addNiceCheckbox(g,"public",group,this);
    protectedAccess=accesspanel.addNiceCheckbox(g,"protected",group,this);
    accesspanel.breakColumn();
    packageAccess=accesspanel.addNiceCheckbox(g,"package",group,this);
    privateAccess=accesspanel.addNiceCheckbox(g,"private",group,this);
    accesspanel.breakColumn();

    BorderedPanel modifpanel=new BorderedPanel(g,master.modifstr,master.infoColor);
    NiceLabel tmp=new NiceLabel(g,"synchronized");
    int w=tmp.prefWidth()+30;
    int h=tmp.prefHeight()+10;
    modifiers=new Choice();
    modifiers.setFont(tmp.getFont());
    //modifiers.setSize(w,h);
    modifiers.addItemListener(this);
    modifpanel.addComponent(modifiers,w,h);
    modifpanel.addComponent(null,w,h);
    printcheck=modifpanel.addNiceCheckbox(g,master.printstr);
    printcheck.setState(true);
    printcheck.addItemListener(this);
    modifpanel.breakColumn();
    group=new CheckboxGroup();
    anyModif=modifpanel.addNiceCheckbox(g,master.anystr,group,this);
    presentModif=modifpanel.addNiceCheckbox(g,master.presentstr,group,this);
    absentModif=modifpanel.addNiceCheckbox(g,master.absentstr,group,this);
    modifpanel.breakColumn();
    modifiers.addItem("abstract");
    modifiers.addItem("static");
    modifiers.addItem("final");
    modifiers.addItem("transient");
    modifiers.addItem("volatile");
    modifiers.addItem("native");
    modifiers.addItem("synchronized");
    NiceCheckbox.makeNice(null,modifiers);

    BorderedPanel sourcepanel=new BorderedPanel(g,master.source,master.infoColor);
    group=new CheckboxGroup();
    javaSrc=sourcepanel.addNiceCheckbox(g,master.javastr,group,this);
    jjSrc=sourcepanel.addNiceCheckbox(g,master.jjstr,group,this);
    htmlcheck=sourcepanel.addNiceCheckbox(g,"HTML",null,this);
    sourcepanel.breakColumn();
    uncompSrc=sourcepanel.addNiceCheckbox(g,master.uncompstr,group,this);
    disasSrc=sourcepanel.addNiceCheckbox(g,master.disasstr,group,this);
    onehtml=sourcepanel.addNiceCheckbox(g,master.onehtmlstr,null,this);
    sourcepanel.breakColumn();
    javaSrc.setState(true);
    htmlcheck.setState(true);//now per default
    onehtml.setState(true);

    BorderedPanel decompilerpanel=new BorderedPanel(g,master.decompilerstr,master.infoColor);
    decompilerChoice=new Choice();
    for(int i=0;i<master.decompiler.length;i++)
	decompilerChoice.addItem(master.decompiler[i].getDecompilerName());
    NiceCheckbox.makeNice(null,decompilerChoice);
    decompilerChoice.addItemListener(this);
    decompilerChoice.setFont(tmp.getFont());
    decompileOptions=new NiceButton(g,master.options);
    decompileOptions.addActionListener(this);

    BorderedPanel sortpanel=new BorderedPanel(g,master.sortstr2,master.infoColor);
    sortcheck=sortpanel.addNiceCheckbox(g,master.sortstr1,null,this);
    sortpanel.breakColumn();
    sortmodif=sortpanel.addNiceCheckbox(g,master.sortmstr,null,this);
    sortpanel.breakColumn();

    Panel all=new Panel(null);
    all.add(namepanel);
    all.add(typepanel);
    all.add(accesspanel);
    all.add(modifpanel);
    all.add(sourcepanel);
    all.add(decompilerpanel);
    all.add(sortpanel);
    all.add(updatePack=new NiceButton(g,master.updatePack));
    all.add(closeButton=new NiceButton(g,master.closemsg));
    add(all);

    //compute the bounds for the layout
    w=namepanel.prefWidth()+typepanel.prefWidth();
    int packw=updatePack.prefWidth();
    int cbw=closeButton.prefWidth()+packw;
    int maxw=Math.max(w,accesspanel.prefWidth());
    int maxw2=Math.max(modifpanel.prefWidth(),sourcepanel.prefWidth());
    int maxw3=sortpanel.prefWidth();
    if (cbw>maxw)
      maxw=cbw;
    if (maxw2>maxw)
      maxw=maxw2;
    if (maxw3>maxw)
      maxw=maxw3;
    //bug if the title is too large
    decompilerpanel.addComponent(decompilerChoice,maxw-2*decompilerpanel.getOffset()-decompileOptions.prefWidth()-BorderedPanel.MIDDLE,h);
    decompilerpanel.addComponent(decompileOptions,decompileOptions.prefWidth(),decompileOptions.prefHeight());
    decompilerpanel.breakColumn();
    //System.out.println("max="+maxw+" offset="+decompilerpanel.getOffset());

    w=(maxw*namepanel.prefWidth())/w;
    namepanel.setBounds(0,0,w,h=namepanel.prefHeight());
    typepanel.setBounds(w,0,maxw-w,typepanel.prefHeight());
    accesspanel.setBounds(0,h,maxw,w=accesspanel.prefHeight());
    h+=w;
    modifpanel.setBounds(0,h,maxw,w=modifpanel.prefHeight());
    h+=w;
    sourcepanel.setBounds(0,h,maxw,w=sourcepanel.prefHeight());
    h+=w;
    decompilerpanel.setBounds(0,h,maxw,w=decompilerpanel.prefHeight());
    h+=w;
    sortpanel.setBounds(0,h,maxw,w=sortpanel.prefHeight());
    h+=w;
    updatePack.setBounds((maxw-cbw)/3,h,packw,updatePack.prefHeight());
    updatePack.addActionListener(this);
    closeButton.setBounds(2*(maxw-cbw)/3+packw,h,closeButton.prefWidth(),closeButton.prefHeight());
    closeButton.addActionListener(this);
    h+=closeButton.prefHeight()+12;
    all.setSize(maxw,h);

    pack();
    //Check size: does not work under some window managers
    //like enlightenment
    Dimension actual=getSize();
    if (actual.width<=maxw)
	actual.width=maxw+10;
    if (actual.height<=h)
	actual.height=h+35;
    setSize(actual);

    setResizable(false);
    Bundle.setLocation(master,this);//set location relative to the parent window
  }

  /**
   * process window events: just close the window when asked
   */
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID()==WindowEvent.WINDOW_CLOSING)
      setVisible(false);
  }

  /**
   * overrides the original in order to update the checkbox statuses
   */
  public void setVisible(boolean visible) {
    if (visible) {
      nameShort.setState(ClassReflect.isShorterNames());
      typeShort.setState(ClassReflect.isShorterTypes());
      switch(ClassReflect.getAccessFilter()) {
      case ClassReflect.ALLACCESS: anyAccess.setState(true); break;
      case Modifier.PUBLIC: publicAccess.setState(true); break;
      case Modifier.PROTECTED: protectedAccess.setState(true); break;
      case Modifier.PRIVATE: privateAccess.setState(true); break;
      case ClassReflect.PACKAGE: packageAccess.setState(true);
      }
      updateModif();
      sortcheck.setState(ClassReflect.isSortEnabled());
      sortmodif.setState(ClassReflect.isSortModifiers());
    }
    super.setVisible(visible);
  }

  /**
   * configure the modifiers checkbox
   */
  void configModif(int flag) {
    switch(flag) {
    case ClassReflect.YES: presentModif.setState(true); break;
    case ClassReflect.NO: absentModif.setState(true); break;
    default: anyModif.setState(true);
    }
  }

  /**
   * action listener
   */
  public void actionPerformed(ActionEvent e) {
    try {
      Object source=e.getSource();
      if (source.equals(closeButton))
	setVisible(false);
      else if (source.equals(updatePack)) {
	PackageFinder.setLabels(null,master.statusLabel,null);
	PackageFinder.updatePackageResources();
	PackageFinder.setLabels(null,null,null);
	master.statusLabel.setText((master.currentClass==null)?"":master.currentClass.toString());
	PackageViewer.updatePackages(getGraphics());
      }
      else if (source.equals(decompileOptions))
	  master.decompiler[master.decompilerID].setupOptions(master);
    }
    catch(Throwable t) {
      t.printStackTrace();
    }
    master.checkstreams();
  }

  /**
   * item listener
   */
  public void itemStateChanged(ItemEvent e) {
    try {
      Object source=e.getSource();
      if (source.equals(sortcheck)) { //sort option
	sortmodif.setEnabled(sortcheck.getState());
	ClassReflect.enableSort(sortcheck.getState());
	master.selectClass();
      }
      else if (source.equals(sortmodif)) { //sort modifiers option
	ClassReflect.enableSortModifiers(sortmodif.getState());
	if (sortmodif.getState())
	  printcheck.setState(true);
	master.selectClass();
      }
      else if (source.equals(javaSrc)) { //java source option
	master.currentSource=null;
	master.sourceOrder=0;
      }
      else if (source.equals(jjSrc)) { //jj (java compiler compiler source) option
	master.currentSource=null;
	master.sourceOrder=1;
      }
      else if (source.equals(uncompSrc)) { //mocha uncompile option
	master.currentSource=null;
	master.sourceOrder=2;
      }
      else if (source.equals(disasSrc)) { //mocha disassemble option
	master.currentSource=null;
	master.sourceOrder=3;
      }
      else if (source.equals(nameShort) || source.equals(nameLong)) { //display options
	ClassReflect.setShorterNames(nameShort.getState());
	master.selectClass();
      }
      else if (source.equals(typeShort) || source.equals(typeLong)) { //display options
	ClassReflect.setShorterTypes(typeShort.getState());
	master.lookAtClass(); //because the super list should be updated
      }
      else if (source.equals(anyAccess)) {
	ClassReflect.setAccessFilter(ClassReflect.ALLACCESS);
	master.selectClass();
      }
      else if (source.equals(publicAccess)) {
	ClassReflect.setAccessFilter(Modifier.PUBLIC);
	master.selectClass();
      }
      else if (source.equals(protectedAccess)) {
	ClassReflect.setAccessFilter(Modifier.PROTECTED);
	master.selectClass();
      }
      else if (source.equals(packageAccess)) {
	ClassReflect.setAccessFilter(ClassReflect.PACKAGE);
	master.selectClass();
      }
      else if (source.equals(privateAccess)) {
	ClassReflect.setAccessFilter(Modifier.PRIVATE);
	master.selectClass();
      }
      else if (source.equals(anyModif)) //any modifier option
	setModif(ClassReflect.ANY);
      else if (source.equals(presentModif)) //requires the modifier
	setModif(ClassReflect.YES);
      else if (source.equals(absentModif)) //requires the absence of the modifier
	setModif(ClassReflect.NO);
      else if (source.equals(modifiers)) //new modifier selected
	updateModif();
      else if (source.equals(decompilerChoice)
	       && e.getStateChange()==e.SELECTED) { //new decompiler choice
	  master.decompilerID=decompilerChoice.getSelectedIndex();
	  master.invalidateSource();//invalidate the current source
	  master.sourceButton.setEnabled(true);
	  decompileOptions.setEnabled(master.decompiler[master.decompilerID].hasOptions());
      }
      else if (source.equals(printcheck)) { //print modifier checkbox
	if (!printcheck.getState()) {
	  ClassReflect.enableSortModifiers(false);
	  sortmodif.setState(false); //for consistency reasons
	}
	master.selectClass();
      } else if (source.equals(htmlcheck))
	  onehtml.setEnabled(htmlcheck.getState());
    }
    catch(Throwable t) {
      t.printStackTrace();
    }
    master.checkstreams();
  }

  /**
   * set the selected modifier
   */
  void setModif(int flag) {
    switch(modifiers.getSelectedIndex()) {
    case 0: ClassReflect.setAbstractFilter(flag);
	//this is a bug of the virtual machine: setResize at
	//the beginning may call this function!
	//new Error("Call with "+flag).printStackTrace();
	break;
    case 1: ClassReflect.setStaticFilter(flag); break;
    case 2: ClassReflect.setFinalFilter(flag); break;
    case 3: ClassReflect.setTransientFilter(flag); break;
    case 4: ClassReflect.setVolatileFilter(flag); break;
    case 5: ClassReflect.setNativeFilter(flag); break;
    case 6: ClassReflect.setSynchronizedFilter(flag);
    }
    master.selectClass();
  }

  /**
   * update the checkboxes for the selected modifier
   */
  void updateModif() {
    switch(modifiers.getSelectedIndex()) {
    case 0: configModif(ClassReflect.getAbstractFilter()); break;
    case 1: configModif(ClassReflect.getStaticFilter()); break;
    case 2: configModif(ClassReflect.getFinalFilter()); break;
    case 3: configModif(ClassReflect.getTransientFilter()); break;
    case 4: configModif(ClassReflect.getVolatileFilter()); break;
    case 5: configModif(ClassReflect.getNativeFilter()); break;
    case 6: configModif(ClassReflect.getSynchronizedFilter());
    }
  }

}




