/**
 * ByteVector
 * a vector just for bytes, the thing is: the
 * Vector class deals only with objects and this
 * one deals with scalars.
 * @version 1.0
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 */
 
package adavid.util;

public class ByteVector {

  /**
   * controls the increment size when a new array has
   * to be allocated.
   */
  public static final int DEFAULT_INC_SIZE=2048;

  /**
   * variables controlling the vector
   */
  int inc_size,size=0;
  byte[] buffer;

  /**
   * contructor: change the default increment size
   */
  public ByteVector(int inc_size) {
    this.inc_size=(inc_size>0)?inc_size:DEFAULT_INC_SIZE;
    buffer=new byte[inc_size];
  }

  /**
   * default constructor
   */
  public ByteVector() {
    this(DEFAULT_INC_SIZE);
  }

  /**
   * add a byte
   */
  public synchronized void add(byte b) {
    if (size>=buffer.length) { //make room
      byte[] bigger=new byte[buffer.length+inc_size];
      System.arraycopy(buffer,0,bigger,0,buffer.length);
      buffer=bigger;
    }
    buffer[size++]=b;
  }
  
  /**
   * returns the size of the vector
   */
  public int length() {
    return size;
  }

  /**
   * returns the byte element at index i
   */
  public synchronized byte byteAt(int i) {
    if (i<0 || i>=size)
      throw new ArrayIndexOutOfBoundsException((i<0)?i+"<0":i+">="+size);
    return buffer[i];
  }

  /**
   * returns an array representation of the byte vector
   */
  public synchronized byte[] toArray() {
    byte[] array=new byte[size];
    System.arraycopy(buffer,0,array,0,size);
    return array;
  }

}
