/**
 * DataResource
 * to have access to private data in resource definition,
 * one should extend this class and override these methods
 * (the same in fact) by copy paste!
 * otherwise, only public data will be accessible
 *
 * to use this class, extend it and override both methods, which
 * should call these ones. That's all, but now, because of the
 * access rights, classes using reflection in the util package
 * have private access right to any class extending this class, via
 * these functions!
 *
 * This is the default implementation of DataResourceAccessible
 * and in fact it is THE implementation that should be followed.
 * @version 1.0
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 */

package adavid.util;
import java.lang.reflect.Field;

public class DataResource implements DataResourceAccessible {

  public void set(Field field,Object value) throws IllegalAccessException {
    field.set(this,value);
  }

  public Object get(Field field) throws IllegalAccessException {
    return field.get(this);
  }

}
