/**
 * JMakefile
 * make java Makefile
 * target is: all .class in the current and in the sub directories
 * @version 1.0.1
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 * 1.0.1 990728: bug fixed: compiler instead of "javac"
 * in the depend rule + some clean up of the code
 * 1.0.2 990801: bug fixed: stupid management of -options
 */

package adavid.util;

import java.io.*;
import java.util.Vector;
import java.util.StringTokenizer;
import java.text.DateFormat;
import java.util.Date;
import java.util.PropertyResourceBundle;

public class JMakefile {

  static String target,date,rootName,compiler;
  static Vector classDir=new Vector();
  static Vector jjDir=new Vector();
  static Vector zgifDir=new Vector();
  static Vector zjpgDir=new Vector();
  static Vector jgifDir=new Vector();
  static Vector jjpgDir=new Vector();
  static boolean deprecation=false,ignore=false;
  static String[] options;

  static int enterDir(File dir,int depth) {
    try {
      Vector dirFound=new Vector();
      Vector classFound=new Vector();
      Vector javaFound=new Vector();
      Vector javaDir=new Vector();
      String[] list=dir.list();
      int ngif=0,njpg=0,subJJ=0;

      System.out.println(dir.getPath());
      for(int i=0;i<list.length;i++) {
	String filename=list[i];
	File file=new File(dir,filename);
	if (file.isDirectory()) {
	  int result;
	  if (filename.charAt(0)!='.' && (result=enterDir(file,depth+1))>=0) {//not hidden and something
	    if (result>0) {
	      dirFound.insertElementAt(filename,0); //this is for jj source
	      javaDir.addElement(filename+".javadir ");
	      subJJ=2; //-1+2=1!
	    }
	    else
	      dirFound.addElement(filename);
	  }
	}
	else if (isExtFile(filename,".java")) {
	  String className=shortName(filename)+".class ";
	  if (!classFound.contains(className))
	    classFound.addElement(className);
	}
	else if (isExtFile(filename,".gif"))
	  ngif++;
	else if (isExtFile(filename,".jpg"))
	  njpg++;
	else if (isExtFile(filename,".jj")) {
	  String className=shortName(filename)+".class ";
	  if (classFound.contains(className))
	    classFound.removeElement(className);
	  classFound.insertElementAt(className,0);
	  javaFound.addElement(shortName(filename)+".java ");
	}
      }

      int nd=dirFound.size();
      int nc=classFound.size();
      int nj=javaFound.size();
      int njd=javaDir.size();

      int index=dir.getPath().indexOf('.');
      if (index<0) {
	System.err.println(dir.getPath());
	throw new Error("Bad path");
      }
      String path=dir.getPath().substring(index+1);

      if (ngif+njpg>0) {
	if (!getInfo(dir,path,zgifDir,zjpgDir,jgifDir,jjpgDir)) {
	  char c;
	  boolean zg=true,zj=true,jg=true,jj=true; //in order to add the directory only once
	  boolean resultzg=false,resultzj=false,resultjg=false,resultjj=false;
	  PrintWriter out=null;
	  try {
	    out=new PrintWriter(new FileWriter(new File(dir,"JMakefile")));
	    out.println("# JMakefile produced by adavid.util.JMakefile\n"+
			"# It saves information requested to take or not the gif/jpg files of this directory\n");
	  }
	  catch(IOException ioe) {
	    System.out.println("JMakefile could not be created!");
	  }
	  if (out!=null || !ignore) {
	    System.out.print("File");
	    if (ngif+njpg>1)
	      System.out.print('s');
	    if (ngif>0)
	      System.out.print(" .gif");
	    if (njpg>0)
	      System.out.print(" .jpg");
	    System.out.print(" found in ["+dir.getPath()+"]\nAdd ["+dir.getPath()+"] in the zip(.java) and jar(.class) lists?(z/j/zj)");
	    do {
	      c=(char)System.in.read();
	      if (c=='z') {
		resultzg=true;//true even if no file, this is by default
		resultzj=true;//fine tuning is obtained by editing the JMakefile
		if (zg && ngif>0) {
		  zgifDir.addElement(path);
		  zg=false;
		}
		if (zj && njpg>0) {
		  zjpgDir.addElement(path);
		  zj=false;
		}
	      }
	      if (c=='j') {
		resultjg=true;//true even if no file, this is by default
		resultjj=true;//fine tuning is obtained by editing the JMakefile
		if (jg && ngif>0) {
		  jgifDir.addElement(path);
		  jg=false;
		}
		if (jj && njpg>0) {
		  jjpgDir.addElement(path);
		  jj=false;
		}
	      }
	    } while (c!='\n');
	    if (out!=null) {
	      out.println("gif.in.jar="+(resultjg&&ngif>0));
	      out.println("gif.in.zip="+(resultzg&&ngif>0));
	      out.println("jpg.in.jar="+(resultjj&&njpg>0));
	      out.println("jpg.in.zip="+(resultzj&&njpg>0));
	      out.flush();
	      out.close();
	      System.out.println("Information saved in "+path+File.separator+"JMakefile");
	    }
	  }
	  else
	    System.out.println(path+" ignored");
	}
      }
     
      if (nc>0)
	classDir.addElement(path);
      if (nj>0)
	jjDir.addElement(path);

      if (nd+nc+nj>0) {
	System.out.print(dir.getPath()+" : ");
	if (nc>0)
	  System.out.print(nc+" class file"+((nc>1)?"s":""));
	if (nc>0 && nd>0)
	  System.out.print(", ");
	if (nd>0)
	  System.out.print(nd+" class director"+((nd>1)?"ies":"y"));
	System.out.println();

	PrintWriter out=new PrintWriter(new FileWriter(new File(dir,"Makefile")));

	out.print("# Automatic Makefile created by adavid.util.JMakefile\n"+
		  "# "+date+
		  "\n# directory : "+dir);
      
	if (depth>0) {
	  String backPath=getBackPath(depth);
	  out.print("\n\nmain :\n\t@echo cd "+backPath+" ; cd "+backPath+" ; make -s");
	}

	out.print("\n\nall : ");
	for(int i=0;i<nd;i++)
	  out.print(dirFound.elementAt(i)+".dir ");
	for(int i=0;i<nc;i++)
	  out.print(classFound.elementAt(i));

	if (depth==0 && target!=null)
	  out.print("\n\t@echo Starting "+target+" ; java "+target);
	out.print("\n\n");

	if (nj+njd>0) {
	  out.print("javasrc : ");
	  for(int i=0;i<njd;i++)
	    out.print(javaDir.elementAt(i));
	  for(int i=0;i<nj;i++)
	    out.print(javaFound.elementAt(i));
	  out.print("\n\n");
	}

	if (depth==0) {
	  int ncd=classDir.size();
	  int nzg=zgifDir.size();
	  int nzj=zjpgDir.size();
	  int njg=jgifDir.size();
	  int njj=jjpgDir.size();
	  int ncj=jjDir.size();

	  if (nc>0) {
	    out.print("javadepend : ");
	    if (njd+nj>0)
	      out.print("javasrc");
	    out.print("\n\t@echo Compiling  ");
	    for(int i=0;i<nc;i++)
	      out.print(rootName+'.'+shortName(classFound.elementAt(i).toString())+" ");
	    out.print(" ; "+compiler+" -depend ");
	    if (deprecation)
	      out.print("-deprecation ");
	    for(int i=0;i<nc;i++)
	      out.print(shortName(classFound.elementAt(i).toString())+".java ");
	    if (target!=null)
	      out.print("; echo Starting "+target+" ; java "+target);
	    out.print("\n\n");
	  }

	  out.print("jdoc :\n\t@cd .. ; ");
	  if (ncj>0) {
	    out.print("jjdoc ");
	    for(int i=0;i<ncj;i++)
	      out.print(rootName+jjDir.elementAt(i)+File.separatorChar+"*.jj ");
	    out.print("; ");
	  }
	  if (ncd>0) {
	    out.print("javadoc ");
	    for(int i=0;i<ncd;i++)
	      out.print(rootName+classDir.elementAt(i)+File.separatorChar+"*.java ");
	  }
	  out.print("\n\n");

	  out.print("zip :\n\t@cd .. ; zip -9 -r "+rootName+"`date +%y%m%d`.zip ");
	  Vector saved=new Vector();
	  for (int i=0;i<ncj;i++) {
	    out.print(rootName+jjDir.elementAt(i)+File.separatorChar+"Makefile ");
	    saved.addElement(jjDir.elementAt(i));
	    out.print(rootName+jjDir.elementAt(i)+File.separatorChar+"*.jj ");
	  }
	  for (int i=0;i<ncd;i++) {
	    if (!saved.contains(classDir.elementAt(i))) {
	      out.print(rootName+classDir.elementAt(i)+File.separatorChar+"Makefile ");
	      saved.addElement(classDir.elementAt(i));
	    }
	    out.print(rootName+classDir.elementAt(i)+File.separatorChar+"*.java ");
	  }
	  if (!saved.contains(""))
	    out.print(rootName+File.separatorChar+"Makefile ");
	  for (int i=0;i<nzg;i++)
	    out.print(rootName+zgifDir.elementAt(i)+File.separatorChar+"*.gif ");
	  for (int i=0;i<nzj;i++)
	    out.print(rootName+zjpgDir.elementAt(i)+File.separatorChar+"*.jpg ");
	  out.print("\n\n");

	  out.print("jar :\n\t@cd .. ; zip -9 -r "+rootName+".jar ");
	  for (int i=0;i<ncd;i++)
	    out.print(rootName+classDir.elementAt(i)+File.separatorChar+"*.class ");
	  for (int i=0;i<njg;i++)
	    out.print(rootName+jgifDir.elementAt(i)+File.separatorChar+"*.gif ");
	  for (int i=0;i<njj;i++)
	    out.print(rootName+jjpgDir.elementAt(i)+File.separatorChar+"*.jpg ");
	  out.print("\n\n");
	}

	out.print("clear : ");
	for(int i=0;i<nd;i++)
	  out.print(dirFound.elementAt(i)+".clear ");
	out.print("\n\t@rm *~\n\n");

	out.print("clearclass : ");
	for(int i=0;i<nd;i++)
	  out.print(dirFound.elementAt(i)+".clearclass ");
	if (nc>0)
	  out.print("\n\t@rm *.class ");
	if (nj>0)
	  for(int i=0;i<nj;i++)
	    out.print(javaFound.elementAt(i));
	out.print("\n\n");     

	if (nj>0)
	  out.print("%.java : %.jj\n\t@echo Generating "+rootName+path+File.separator+"$*.java ; javacc $<\n\n");
	if (njd>0)
	  out.print("%.javadir : %\n\t@cd $< ; make -s javasrc\n\n");
	if (nc>0) {
	  out.print("%.class : %.java\n\t@echo Compiling "+rootName+path.replace(File.separatorChar,'.')+".$* ; "+compiler+" ");
	  if (deprecation)
	    out.print("-deprecation ");
	  for(int i=0;i<options.length;i++)
	      out.print(options[i]+" ");
	  out.print("$<\n\n");
	}
	if (nd>0)
	  out.print("%.dir : %\n\t@cd $< ; make -s all\n\n"+
		    "%.clear : %\n\t@echo cd $< ; cd $< ; make -s -i clear ; echo cd ..\n\n"+
		    "%.clearclass : %\n\t@echo cd $< ; cd $< ; make -s -i clearclass ; echo cd ..\n\n");

	out.flush();
	out.close();
	return nj+subJJ;
      }
      else
	return -1+subJJ;
    }
    catch(Exception e) {
      System.err.println(e);
      return -1;
    }
  }

  static boolean getInfo(File dir,String path,
			 Vector zgifDir,Vector zjpgDir,
			 Vector jgifDir,Vector jjpgDir) {
    File jmakeInfo=new File(dir,"JMakefile");
    try {
      PropertyResourceBundle bundle=
	new PropertyResourceBundle(
	  new BufferedInputStream(
	    new FileInputStream(jmakeInfo)));
      ResourceLoader loader=new ResourceLoader(bundle);
      String def=loader.FALSE;
      if (loader.getBoolResource("gif.in.jar",def))
	jgifDir.addElement(path);
      if (loader.getBoolResource("gif.in.zip",def))
	zgifDir.addElement(path);
      if (loader.getBoolResource("jpg.in.jar",def))
	jjpgDir.addElement(path);
      if (loader.getBoolResource("jpg.in.zip",def))
	zjpgDir.addElement(path);
      return true;
    }
    catch(IOException ioe) {
      System.out.println("Information file JMakefile not found!");
      return false;
    }
  }

  /**
   * returns a back path of the form ../../../ corresponding to depth
   */
  public static String getBackPath(int depth) {
    if (depth<1)
      return ".";
    StringBuffer buffer=new StringBuffer();
    for (int i=0;i<depth;i++)
      buffer.append("..").append(File.separatorChar);
    return buffer.toString();
  }

  /**
   * returns true if the string name ends with extension ext
   */
  public static boolean isExtFile(String name,String ext) {
    int index=name.lastIndexOf('.');
    if (index<0)
      return false;
    else
      return name.substring(index).equals(ext);
  }

  /**
   * returns the short name of the string name, which is the name without
   * the last extension beginning with a dot.
   */
  public static String shortName(String name) {
    int index=name.lastIndexOf('.');
    if (index<0)
      return name;
    else
      return name.substring(0,index);
  }

  /**
   * main program, run it with nothing to know the options and how to use it
   */
  public static void main(String[] args) {
    long now=System.currentTimeMillis();
    Date dateNow=new Date(now);
    File dir=new File(".");
    StringTokenizer tokenizer=new StringTokenizer(dir.getAbsolutePath(),"."+File.separatorChar);
    CommandSorter command=new CommandSorter(args);
    options=command.getArgsAfter("-options");
    compiler=command.getDefinition("-javac");
    if (compiler==null)
      compiler="javac";

    if (args.length==0)
      System.out.println("possible options: java adavid.util.JMakefile [target] [-deprecation] [-ignore]"+
			 " [-javac=newcompiler] [-options [java options]]\n"+
			 "where -deprecation activates the -deprecation option of javac\n"+
			 "      -javac=newcompiler gives a new compiler to use, but not for the -depend options\n"+
			 "      -ignore says to ignore any gif/jpg directory which is write protected\n"+
			 "      target is the class target to be run by java after the compilation\n"+
			 "      -options gives options to the java compiler");

    deprecation=command.isDefined("-deprecation");
    ignore=command.isDefined("-ignore");
    args=command.getAllArgs();
    if (args.length>0)
      target=args[0];

    date=DateFormat.getDateInstance(DateFormat.FULL).format(dateNow);
    rootName=tokenizer.nextToken();
    while(tokenizer.hasMoreTokens())
      rootName=tokenizer.nextToken();

    enterDir(dir,0);
  }
}
