/**
 * Observatory
 * This class uses Observer object to notify
 * but we do not need Observable object. What
 * I do not like with Observable is that it is
 * a class and if we have some custom-special
 * class to be observed, then we have to create
 * some observable linked to the class... and I
 * do not like it specially because the setChanged()
 * is protected, which means that we have to derive
 * the Observer class in order to use it!
 * @version 1.0
 * @author Alexandre David
 */

/**
 * version history
 * 1.0: first version
 */

package adavid.util;

import java.util.*;

public class Observatory {

  /**
   * register class which want to update other objects
   */
  private static Hashtable registration=new Hashtable();

  /**
   * register an Updateable object
   */
  static public void register(Object observable,Updateable observer) {
    Vector list;
    synchronized(registration) {
      list=(Vector)registration.get(observable);
      if (list==null) {
	list=new Vector();
	registration.put(observable,list);
      }
    }
    //the list may change between, it does not matter because
    //the list is created once, then we add/remove elements
    synchronized(list) {
      if (!list.contains(observer))
	list.addElement(observer);
    }
  }

  /**
   * notify observers
   */
  static public void notify(Object observable,Object arg) {
    Vector list;
    synchronized(registration) {
      list=(Vector)registration.get(observable);
      if (list==null)
	return;
    }
    Updateable[] all;
    synchronized(list) {
      all=new Updateable[list.size()];
      list.copyInto(all);
    }
    for(int i=0;i<all.length;i++)
      all[i].update(observable,arg);
  }

  /**
   * unregister an Updateable object
   */
  static public void unregister(Object observable,Updateable observer) {
    Vector list;
    synchronized(registration) {
      list=(Vector)registration.get(observable);
      if (list==null)
	return;
    }
    //the list may change between, it does not matter because
    //the list is created once, then we add/remove elements
    synchronized(list) {
      list.removeElement(observer);
    }
  }

  /**
   * unregister all Updateable objects of an observable
   */
  static public void unregister(Object observable) {
    synchronized(registration) {
      registration.remove(observable);
    }
  }

}
