/**
 * PackageViewer main program
 * @author Alexandre David
 * @version 1.0.2
 */

/**
 * version history
 * 1.0: first version
 * 1.0.1 990731: little bug fix: in the class list, when
 *        only one class was displayed there
 *        was "1 classes", now it's "1 class"
 * 1.0.2 990905: increment steps for scrollbars
                 tried and abandonned cache package classes
 */

package adavid.util;

import java.awt.*;
import adavid.awt.*;
import adavid.reflect.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Hashtable;
import java.lang.reflect.Modifier;

/**
 * PackageViewer
 * view all the available packages and gives
 * simple information on classes, send
 * a class to Assistant for more information
 */
public class PackageViewer extends Frame implements TListener,ActionListener,TNodeRenderer,ItemListener {

  /**
   * version and final constants
   */
  public static final String VERSION="PackageViewer 1.0.2";
  public static final Cursor HAND=Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);

  /**
   * instance of this class, one instance is enough
   * per VM
   */
  static PackageViewer viewer;

  /**
   * graphical components: the labels used to display
   * information and titles, the scroll panels, the stuff
   * for both trees, the list for simple information and
   * the close and call assistant buttons. resources contains
   * information on the packages which are installed and
   * accessible from the java VM, initialized by PackageFinder
   */
  NiceLabel statusLabel,packageLabel,classLabel;
  PackageResource[] resources;
  ScrollPane packageScroll,classScroll;
  TNodeRenderer classRenderer;
  ImageTNodeRenderer packRenderer=new ImageTNodeRenderer(ImageTNodeRenderer.DIR_ONLY);
  TNode packRoot,classRoot;
  List infoList=new List(3);
  NiceButton closeButton,assistButton;
  Dimension dimIcons=new Dimension(); //dimension of the icons
  String[] classKeys,classNames;

    /**
     * cache the queried packages: store a vector -> title and nodes
     */
    //Hashtable hashPackage=new Hashtable(); //little gain and display problem!

  /**
   * the constructor: here we do the job!
   * protected in case someone wants to extends
   * the features.
   */
  protected PackageViewer() {
    super(VERSION);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    setLayout(new BorderLayout());
    setBackground(Color.lightGray);
    setSize(530,350);
    classRenderer=this;

    //the center panel: gridlayout=one for the packages , one for the classes
    Panel centerpanel=new Panel(new GridLayout(1,0));

    //package panel
    Panel packpanel=new Panel(new BorderLayout());
    packRoot=new TNode(null);
    packRoot.setOpenOnDoubleClick(false);
    packRoot.setMutable(false);
    packageScroll=new ScrollPane();
    packageScroll.add(packRoot);
    packRoot.setScrollPane(packageScroll);
    packRoot.setRenderer(packRenderer);
    packRoot.expand();
    packRoot.addTListener(this);
    packpanel.add(packageScroll,BorderLayout.CENTER);

    //class panel
    Panel classpanel=new Panel(new BorderLayout());
    classRoot=new TNode(null);
    classRoot.setOpenOnDoubleClick(false);
    classRoot.setMutable(false);
    classScroll=new ScrollPane();
    classScroll.add(classRoot);
    classRoot.setScrollPane(classScroll);
    classRoot.setRenderer(classRenderer);
    classRoot.expand();
    classRoot.addTListener(this);
    classpanel.add(classScroll,BorderLayout.CENTER);
    classpanel.add(infoList,BorderLayout.SOUTH);
    
    centerpanel.add(packpanel);
    centerpanel.add(classpanel);
    add(centerpanel,BorderLayout.CENTER);
    Bundle.show(this);

    Graphics g=getGraphics();
    if (g==null) {
      System.err.println("Panic: getGraphics()==null");
      System.exit(1);
    }

    //intialize the labels and add the status label
    statusLabel=new NiceLabel(g,"");
    statusLabel.setForeground(Assistant.infoColor);
    packageLabel=new NiceLabel(g,"");
    classLabel=new NiceLabel(g,"");
    add(statusLabel,BorderLayout.SOUTH);
    packpanel.add(packageLabel,BorderLayout.NORTH);
    classpanel.add(classLabel,BorderLayout.NORTH);

    //south subpanel of the package panel: close window and send to Assistant
    Panel southpack=new Panel(new BorderLayout());
    packpanel.add(southpack,BorderLayout.SOUTH);
    southpack.add(closeButton=new NiceButton(g,Assistant.closemsg),BorderLayout.WEST);
    closeButton.addActionListener(this);
    southpack.add(assistButton=new NiceButton(g,""),BorderLayout.CENTER);
    assistButton.addActionListener(this);
    doLayout();
    centerpanel.doLayout();
    classpanel.doLayout();
    packpanel.doLayout();
    southpack.doLayout();

    initPackageTree(g);
    /******** icon stuff *********/
    dimIcons.width=0;
    dimIcons.height=0;
    ImageTNodeRenderer.lookDim(Assistant.FileI,dimIcons);
    ImageTNodeRenderer.lookDim(Assistant.FileISelected,dimIcons);
    ImageTNodeRenderer.lookDim(Assistant.FileA,dimIcons);
    ImageTNodeRenderer.lookDim(Assistant.FileASelected,dimIcons);
    dimIcons.width+=ImageTNodeRenderer.SIZE+2;
    if (dimIcons.height<ImageTNodeRenderer.SIZE+1)
      dimIcons.height=ImageTNodeRenderer.SIZE+1;
    //this icon stuff is done because this class is a TNodeRenderer
    //which is used in the class tree: abstract, interface, and normal
    //classes, selected or not
    packageScroll.getVAdjustable().setUnitIncrement(dimIcons.height+2);
    classScroll.getVAdjustable().setUnitIncrement(dimIcons.height+2);

    infoList.addItemListener(this);
    infoList.setCursor(HAND);
    AutoFocuser.add(infoList);
  }

    /**
     * setup a new root for the class tree
     *
    void setClassRoot() {
	classRoot=new TNode(null);
	classRoot.setOpenOnDoubleClick(false);
	classRoot.setMutable(false);
	classScroll.removeAll();
	classScroll.add(classRoot);
	classRoot.setScrollPane(classScroll);
	classRoot.setRenderer(classRenderer);
	classRoot.expand();
	classRoot.addTListener(this);
     }*/ //little gain and display problem!

  /**
   * initialize the package tree
   */
  public synchronized void initPackageTree(Graphics g) {
    int total=0;
    //PackageFinder may display information while running
    //if its labels are set to non null values.
    PackageFinder.setLabels(null,statusLabel,null);
    packageScroll.setVisible(false); //## check that this is in TNode
    resources=PackageFinder.getPackageResources();
    if (resources.length>1)
      Sorter.sortI(resources,0,resources.length-1);
    PackageFinder.setLabels(null,null,null);
    packRoot.removeAllNodes();
    //System.out.println(Sorter.toString(resources));

    int packnb=0;
    StringBuffer buffer=new StringBuffer();
    Vector listkeys=new Vector(5000),listnames=new Vector(5000);
    //we store all the possible class names in listnames and we
    //store all possible matches (lower case) for class search.
    for(int i=0;i<resources.length;i++) {
      statusLabel.setText(resources[i].toString());
      String[] packageclasses=resources[i].getClassNames();
      int nbclasses=packageclasses.length;
      //System.out.println(resources[i]+" has "+nbclasses+" classes");
      if (nbclasses>0) {
	total+=nbclasses;
	packnb++;
	TNode node=putInTree(resources[i],packRoot,g);
	node.getComponent().setCursor(HAND);
	//store the keys and the names
	//manual management of StringBuffer because this is compiled
	//in a similar way by javac though not so cleanly.
	buffer.setLength(0);
	String prefix=(resources[i].pack.equals(""))?"":buffer.append(resources[i].pack).append('.').toString();
	for(int j=0;j<nbclasses;j++) {
	  listkeys.addElement(packageclasses[j].toLowerCase());
	  buffer.setLength(0);
	  listnames.addElement(buffer.append(prefix).append(packageclasses[j]).toString());
	}
      }
    }
    packRoot.invalidateNode();
    classNames=new String[total];
    classKeys=new String[total];
    listnames.copyInto(classNames);
    listkeys.copyInto(classKeys);
    packageScroll.setVisible(true);
    //System.out.println(adavid.util.Sorter.toString(classKeys));

    statusLabel.setText("");
    buffer.setLength(0); //reuse is good and cheap
    packageLabel.setText(buffer.append(packnb).append(Assistant.packstr2).append(',').append(total).append(Assistant.classstr).toString());
  }

  /**
   * return a list of possible corrections for a class name
   */
  public String[] getPropositions(String name) {
    Vector result=new Vector(30);
    int n=classKeys.length;
    for(int i=0;i<n;i++)
      if (classKeys[i].indexOf(name)>=0)
	result.addElement(classNames[i]);
    String[] list=new String[result.size()];
    result.copyInto(list);
    return Sorter.matchSort(name,list,false); //special sort: nearest match order
    //return list;
  }

  /**
   * put a node in the tree
   */
  TNode putInTree(PackageResource res,TNode root,Graphics g) {
    String name=Bundle.shortName(res.pack);
    if (name.equals(""))
      name="        "; //to have a non empty clickable zone
    TNode node=new TNode(res,new FLabel(g,name));
    AutoFocuser.add(node.getComponent());
 
    //## check that there is a hide somewhere, or if it is possible to
    //## do it more efficiently
    String pack=res.pack;
    int index;
    while((index=pack.indexOf('.'))>0) {
      String subpack=pack.substring(0,index);
      pack=pack.substring(index+1);

      TNode[] sub=root.getSubNodes();
      int i=0;
      while(i<sub.length && !sub[i].getStringLabel().equals(subpack)) i++;
      
      if (i==sub.length) { //not found -> create
	TNode subnode=new TNode(subpack,new FLabel(g,subpack));
	AutoFocuser.add(subnode.getComponent());
	statusLabel.setText(subpack);
	root.addNode(subnode);
	root=subnode;
      }
      else
	root=sub[i];
      //System.out.print(root.getStringLabel()+" -> ");
    }
    statusLabel.setText(name);
    root.addNode(node);
    return node;
    //System.out.println(node.getStringLabel());
  }

  /**
   * select a package->show its classes
   */
  public void selectPackage(PackageResource pack) {
    classScroll.setVisible(false);
    classRoot.removeAllNodes(); //commented if cached
    if (infoList.getItemCount()>0)
      infoList.removeAll();
    //Object cached=hashPackage.get(pack);
    //if (cached==null) { //not stored
    //Vector cacheline=new Vector();
	int nb=pack.getClasses().length;
	String packTitle=pack.pack+":"+nb+((nb>1)?Assistant.classstr:Assistant.oneclassstr);
	//setClassRoot();
	classLabel.setText(packTitle);
	//cacheline.addElement(packTitle);
	//cacheline.addElement(classRoot);
	Class[] classes=pack.getTopClasses();
	for(int i=0;i<classes.length;i++)
	    addClass(classRoot,classes[i],pack);
	//hashPackage.put(pack,cacheline);
	//} else {
	//Vector cacheline=(Vector)cached;
	//classLabel.setText(cacheline.firstElement().toString());
	//classRoot=(TNode)cacheline.elementAt(1);
	//classScroll.removeAll();
	//classScroll.add(classRoot);
	//}
    classRoot.invalidateNode();
    classScroll.setVisible(true);
    statusLabel.setText("");
    assistButton.setLabel("");
  }

  /**
   * add a class to the class hierarchy, and all its sub classes too
   * the algorithm is not very performant, I'll change it later, but
   * the limiting factor is the graphical implementation.
   */
  void addClass(TNode root,Class theclass,PackageResource pack) {
    String name=Bundle.shortName(theclass.getName());
    TNode newnode=new TNode(theclass,new FLabel(getGraphics(),name));
    AutoFocuser.add(newnode.getComponent());
    newnode.getComponent().setCursor(HAND);
    statusLabel.setText(name);
    root.addNode(newnode);
    Class[] extensions=pack.extensionsOf(theclass);
    for(int i=0;i<extensions.length;i++)
      addClass(newnode,extensions[i],pack);
    if (extensions.length==0)
      newnode.expand();
  }

  /**
   * TListener interface implementation
   */
  public void nodeDoubleClicked(TNode node) {
    Object selected=node.getContent();
    if (selected instanceof Class)
      Assistant.queryClass(((Class)selected).getName());
    else if (selected instanceof PackageResource) {
      selectPackage((PackageResource)selected);
      statusLabel.setText(selected.toString());
    }
  }
  public void nodeClicked(TNode node) {
    Object selected=node.getContent();
    if (selected instanceof PackageResource)
      statusLabel.setText(selected.toString());
    else if (selected instanceof Class) {
      Class theclass=(Class)selected;
      //System.out.println(selected);
      Class superclass=theclass.getSuperclass();
      Class[] interf=theclass.getInterfaces();
      if (infoList.getItemCount()>0)
	infoList.removeAll();
      infoList.addItem((theclass.isInterface()?"interface ":"class ")+ClassReflect.getString(theclass));
      if (superclass!=null)
	infoList.addItem("extends "+ClassReflect.getString(superclass));
      for(int i=0;i<interf.length;i++)
	infoList.addItem("implements "+ClassReflect.getString(interf[i]));
      assistButton.setLabel(theclass.getName());
    }
    else
      statusLabel.setText("");
  }

  /**
   * ActionListener implementation
   */
   public void actionPerformed(ActionEvent e) {
     Object source=e.getSource();
     if (source.equals(closeButton))
       setVisible(false);
     else if (source.equals(assistButton) && !assistButton.getLabel().equals(""))
       Assistant.queryClass(assistButton.getLabel());
   }

  /**
   * process window events: just close the window when asked
   */
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID()==WindowEvent.WINDOW_CLOSING)
      setVisible(false);
  }

  /**
   * call the package viewer
   */
  public static void callViewer() {
    if (viewer==null)
      viewer=new PackageViewer();
    else
      viewer.setVisible(true);
  }

  /**
   * update the package viewer
   */
  public static void updatePackages(Graphics g) {
    if (viewer!=null)
      viewer.initPackageTree(g);
  }

  /**
   * why not having this class as an application?
   */
  public static void main(String argv[]) {
    callViewer();
  }

  /**
   * we render the node here: draw the icon and the triangle
   * note that the TNode chooses the color of the triangle
   * this is for the TNodeRenderer
   */
  public void paint(TNode c,Graphics g,boolean isExpanded,boolean isSelected,boolean isLeaf) {
    Image icon;
    Object object=c.getContent();
    if (object!=null) {
      Class theclass=(Class)object;
      int modif=theclass.getModifiers();
      if (Modifier.isInterface(modif))
	icon=(isSelected)?Assistant.FileISelected:Assistant.FileI;
      else if (Modifier.isAbstract(modif))
	icon=(isSelected)?Assistant.FileASelected:Assistant.FileA;
      else
	icon=(isSelected)?ImageTNodeRenderer.TreeLeafSelected:ImageTNodeRenderer.TreeLeaf;
    }
    else
      icon=(isSelected)?ImageTNodeRenderer.TreeLeafSelected:ImageTNodeRenderer.TreeLeaf;
    if (!isLeaf) {
      if (isExpanded)
	g.fillPolygon(ImageTNodeRenderer.pictx,ImageTNodeRenderer.picty,3);
      else
	g.fillPolygon(ImageTNodeRenderer.picty,ImageTNodeRenderer.pictx,3);
    }
    //System.out.println("icon:"+icon);
    if (icon!=null)
      g.drawImage(icon,ImageTNodeRenderer.SIZE+1,0,icon.getWidth(null),icon.getHeight(null),c);
  }

  /**
   * implements ItemListener
   */
  public void itemStateChanged(ItemEvent e) {
    if (e.getSource().equals(infoList)) {
      String item=infoList.getSelectedItem();//can not be null
      assistButton.setLabel(item.substring(item.indexOf(' ')+1));
    }
  }

  /**
   * the dimension taken by the rendered image and triangle ;
   * updateDim() is assumed to be called before ; has to be done
   * explicitely if one changes one icon ; implements TNodeRenderer
   */
  public int getHeight() { return dimIcons.height; }
  public int getWidth()  { return dimIcons.width;  }
}
