/**
 * MochaGate
 * use mocha thanks to reflection
 * @author Alexandre David
 * @version 1.0.2
 */

package mocha;

/**
 * 1.0: first version
 * 1.0.1: 990902 slight windoze fix
 * 1.0.2: 990905 added interface Decompiler
 * 1.1: 990922 added interface ItemListener and
 * option configuration
 */

import java.lang.reflect.*;
import adavid.io.*;
import java.io.*;
import adavid.reflect.*;
import adavid.awt.*;
import adavid.util.*;
import java.awt.*;
import java.awt.event.*;

public class MochaGate implements adavid.reflect.Decompiler,ItemListener {

    /**
     * components related to the option window
     */
    NiceCheckbox verboseCheck,debugCheck;
    DialogFactory optionsDialog;
    Assistant master;

  /**
   * if raw is true, we have java assembly code!
   */
  public static boolean raw=false;

  /**
   * create a mocha.3 object
   */
  static Object create3(InputStream is) throws Exception {
    Class[] arg=new Class[] { Class.forName("java.io.InputStream") };
    Constructor cons=Class.forName("mocha.3").getDeclaredConstructor(arg);
    //System.out.println("Constructor "+cons+" created");
    return cons.newInstance(new Object[] { is });
  }

  /**
   * create a mocha.0 object
   */
  static Object create0(Object obj3,String filename) throws Exception {
    Class[] arg=new Class[] { Class.forName("mocha.3") , Class.forName("java.lang.String") };
    Constructor cons=Class.forName("mocha.0").getDeclaredConstructor(arg);
    //System.out.println("Constructor "+cons+" created");
    return cons.newInstance(new Object[] { obj3 , filename });
  }

  /**
   * create a mocha.123 object
   */
  static Object create123(OutputStream os) throws Exception {
    Class[] arg=new Class[] { Class.forName("java.io.OutputStream") };
    Constructor cons=Class.forName("mocha.123").getDeclaredConstructor(arg);
    //System.out.println("Constructor "+cons+" created");
    return cons.newInstance(new Object[] { os });
  }

  /**
   * target invokes a method className.methodName(void)
   */
  static void voidMethod(Object target,String className,String methodName) throws Exception {
    Method method=Class.forName("mocha."+className).getMethod(methodName,new Class[0]);
    //System.out.println("mocha."+className+"."+methodName+"() created");
    method.invoke(target,new Object[0]);
  }

  /**
   * decompile from a class
   */
  public static String decompile(Class c) {
    return decompile(c.getName());
  }

  /**
   * open a class stream (forbidden by ClassLoader!)
   */
  public static InputStream openStream(String packName,String className) throws Exception {
    //System.out.println("MochaGate.openStream("+packName+","+className+")");
    PackageResource[] resources=PackageFinder.getPackageResources(packName);
    for(int i=0;i<resources.length;i++) {
      InputStream is=resources[i].openStream(className);
      //if (is!=null) System.out.println("return "+is);
      if (is!=null)
	return is;
      //if (is!=null)
      //return copyFile(is);
    }
    //System.out.println("Class not found!");
    throw new Exception(packName+'.'+className+" not found!");
  }

  /*static InputStream copyFile(InputStream is) throws Exception {
    String filename=System.getProperty("user.home")+File.separator+".MochaGate.tmp";
    FileOutputStream fous=new FileOutputStream(filename);
    int i;
    while((i=is.read())>=0)
      fous.write(i);
    is.close();
    fous.close();
    return new FileInputStream(filename);
  }*/

  /**
   * decompile from a class name (absolute name)
   */
  public static String decompile(String className) {
    try {
      int index=className.lastIndexOf('.');
      InputStream is=openStream(className.substring(0,(index<0)?0:index),
				className.substring(index+1));

      //System.out.println(is);
      Object obj3=create3(new BufferedInputStream(is));
      //System.out.println("mocha.3 created");
      Object obj0=create0(obj3,className.replace('.','/'/*File.separatorChar*/)+".class");
      //System.out.println("mocha.0 created");
      voidMethod(obj3,"3","close");
      //System.out.println("mocha.3.close() done");
      if (!raw) {
	voidMethod(obj0,"0","39");
	//System.out.println("mocha.0.39() done");
      }
      StringOutputStream sos=new StringOutputStream();
      Object obj123=create123(new BufferedOutputStream(sos));
      //System.out.println("mocha.123 created");
      Class[] arg=new Class[] { Class.forName("java.io.PrintStream") };
      Method method=Class.forName("mocha.0").getMethod("17",arg);
      //System.out.println("mocha.0.17(PrintStream) created");
      PrintStream printStream=new PrintStream((OutputStream)obj123);
      method.invoke(obj0,new Object[] { printStream });
      //System.out.println("mocha.0.17(PrintStream() done");
      printStream.close();
      return sos.toString();
    }
    catch(InvocationTargetException ite) {
      System.err.println("mocha.MochaGate.decompile("+className+"):");
      ite.getTargetException().printStackTrace();
      return null;
    }
    catch(Exception e) {
      System.err.println("mocha.MochaGate.decompile("+className+"):");
      e.printStackTrace();
      return null;
    }
  }

  /**
   * test: argument = class name
   * options : -v (verbose) -d (debug) -r (raw)
   */
  public static void main(String[] args){

    String className=null;
    for(int i=0;i<args.length;i++)
      if (args[i].equals("-v"))
	Decompiler.verbose=true;
      else if (args[i].equals("-d"))
	Decompiler.debug=true;
      else if (args[i].equals("-r"))
	raw=true;
      else
	className=args[i];

    if (className==null) {
      System.err.println("Usage: java mocha.MochaGate [-v][-d][-r] className");
      System.exit(1);
    }

    //save the standard output and error streams
    PrintStream out0=System.out;
    PrintStream err0=System.err;

    //pseudo output stream->to string
    StringOutputStream out1=new StringOutputStream();
    StringOutputStream err1=new StringOutputStream();

    //redirect the outputs
    System.setOut(new PrintStream(out1));
    System.setErr(new PrintStream(err1));

    //uncompile
    String result=decompile(className);

    //restore the outputs
    System.setOut(out0);
    System.setErr(err0);

    //print the results
    System.out.println(result);
    System.out.println("Out message:\n"+out1);
    System.err.println("Err message:\n"+err1);

  }

    //---------------- interface Decompile ---------------

    /**
     * set decompile mode
     */
    public void setDecompile() {
	raw=false;
    }

    /**
     * set disassemble mode
     */
    public void setDisassemble() {
	raw=true;
    }

    /**
     * identification
     */
    public String getDecompilerName() {
	return "Mocha";
    }

    /**
     * decompile
     */
    public String decompileClass(Class className) {
	return decompile(className);
    }

    /**
     * setup options
     */
    public void setupOptions(Assistant master) {
	if (optionsDialog==null) {
	    Graphics g=master.getGraphics();
	    this.master=master;
	    optionsDialog=DialogFactory.getDialog(master,Assistant.infoColor,
						  getDecompilerName(),
						  Assistant.options,
						  new NiceCheckbox[] { verboseCheck=new NiceCheckbox(g,Assistant.verbosestr) , debugCheck=new NiceCheckbox(g,Assistant.debugstr) });
	    verboseCheck.addItemListener(this);
	    debugCheck.addItemListener(this);
	}
	optionsDialog.setVisible(!optionsDialog.isVisible());
    }

    /**
     * implements ItemListener
     */
    public void itemStateChanged(ItemEvent e) {
	Decompiler.verbose=verboseCheck.getState();
	Decompiler.debug=debugCheck.getState();
	master.invalidateSource();
	//System.out.println("mocha options");
    }
    
    /**
     * @return true since it has some options to
     * be configured
     */
    public boolean hasOptions() {
	return true;
    }

}
